// BusyState

#include "BusyState.h"
#include <AppKit/AppKit.h>
#include "../General.bproj/UniqueString.h"

NSString *BusyStateEntityType;

@implementation BusyState

static NSMutableDictionary *colorDict;

+ (void)initialize
{
    if (self == [BusyState class]) {
        BusyStateEntityType = U(@"Activity State");
    }
}

+ (NSMutableDictionary *)colorDict
{
    if (!colorDict)
        colorDict = [[NSMutableDictionary alloc] init];
    return colorDict;
}

+ (BusyState *)stateWithStartTime:(NSDate *)start
                          endTime:(NSDate *)end
                        container:(PajeContainer *)cont
                  relatedEntities:(NSArray *)related
{
    return [[[self alloc] initWithStartTime:start
                                    endTime:end
                                  container:cont
                            relatedEntities:related] autorelease];
}

- (id)initWithStartTime:(NSDate *)start
                endTime:(NSDate *)end
              container:(PajeContainer *)cont
        relatedEntities:(NSArray *)related
{
    self = [super init];
    startTime = [start retain];
    endTime = [end retain];
    container = [cont retain];
    relatedEntities = [related retain];
    return self;
}

- (void)dealloc
{
    [startTime release];
    [endTime release];
    [container release];
    [relatedEntities release];
    [super dealloc];
}


+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", nil];
    return hier;
}

- (id)container { return container; }

- (NSDate *)startTime  { return startTime; }
- (NSDate *)endTime    { return endTime;   }
+ (PajeDrawingType)drawingType { return PajeVariableDrawingType; } //PajeStateDrawingType; }
- (PajeDrawingType)drawingType { return PajeVariableDrawingType; } //PajeStateDrawingType; }
- (NSNumber *)value {
    return [NSNumber numberWithInt:[relatedEntities count]];
}
- (NSString *)entityType { return BusyStateEntityType;}
+ (NSString *)entityType { return BusyStateEntityType;}
+ (NSString *)name { return BusyStateEntityType;}
- (NSString *)name
{
    unsigned ct = [relatedEntities count];
    if (ct == 0) return @"Inactive";
    if (ct == 1) return @"1 active thread";
    return [NSString stringWithFormat:@"%d active threads", ct];
}

- (NSColor *)color
{
    int ct = [relatedEntities count];
    if (ct >= 10) return [NSColor redColor];
    return [[NSColor blueColor] blendedColorWithFraction:ct/10. ofColor:[NSColor redColor]];
    return [NSColor colorWithDeviceHue:.667 - ct/15. saturation:1.0 brightness:1.0 alpha:1.0];
}

- (NSArray *)relatedEntities { return relatedEntities; }

/*
- (void)inspect
{
    NSLog(@"%@ inspection:\nstartTime=%@\nendTime=%@\nrelatedEntities=%@",
          NSStringFromClass([self class]), startTime, endTime, relatedEntities);
}
*/
@end
