#ifndef _ContainerSelector_h_
#define _ContainerSelector_h_

/* ContainerSelector.h created by benhur on Sat 03-Jul-2004 */

#include <AppKit/AppKit.h>
#include "../General.bproj/FilteredEnumerator.h"
#include "../General.bproj/Protocols.h"
#include "../General.bproj/PajeFilter.h"

@interface ContainerSelector : PajeFilter
{
    NSMutableDictionary *filters;
    NSMutableSet *hiddenEntityTypes;
    IBOutlet NSView *view;
    IBOutlet NSMatrix *matrix;
    IBOutlet NSPopUpButton *entityTypePopUp;
    IBOutlet NSButton *hideButton;
}

- (id)initWithController:(PajeTraceController *)c;
- (void)dealloc;

- (void)setFilterForEntityType:(PajeEntityType *)entityType;

- (PajeEntityType *)selectedEntityType;

// read and write defaults
- (NSMutableSet *)defaultFilterForEntityType:(PajeEntityType *)entityType;
- (void)registerDefaultFilter:(NSSet *)filter
                forEntityType:(PajeEntityType *)entityType;

//
// Handle interface messages
//
- (IBAction)selectAll:(id)sender;
- (IBAction)unselectAll:(id)sender;
- (IBAction)matrixChanged:(id)sender;
- (IBAction)entityPopUpChanged:(id)sender;
- (IBAction)hideButtonClicked:(id)sender;

//
// interact with interface
//
- (void)viewWillBeSelected:(NSView *)view;
- (void)synchronizeMatrix;


//
// methods in Trace for making the filtering
//
- (NSEnumerator *)enumeratorOfContainersTyped:(PajeContainerType *)entityType
                                  inContainer:(PajeContainer *)container;

- (BOOL)isHiddenEntityType:(PajeEntityType*)entityType;

//
// methods to be a dragging destination, as delegate of matrix
// (colors dropped in a cell change the color of the 
// entity type it represents)
//
- (unsigned int)matrix:(NSMatrix *)m draggingUpdated:(id <NSDraggingInfo>)sender;
- (BOOL)matrix:(NSMatrix *)m performDragOperation:(id <NSDraggingInfo>)sender;
- (void)matrix:(NSMatrix *)m draggingExited:(id <NSDraggingInfo>)sender;
// help method for highlighting the destination cell
- (void)highlightCell:(id)cell inMatrix:(NSMatrix *)m;

@end

#endif
