#ifndef _NSMatrix_Additions_h_
#define _NSMatrix_Additions_h_

// NSMatrix+Additions.h
//
// NSMatrix (NSDraggingDestination)
// Category to NSMatrix that allows it's delegate to receive
// dragging destination messages.
// To receive draggingEntered: messages, the delegate has to implement
// -matrix:(NSMatrix *)m draggingEntered:(id <NSDraggingInfo>)sender
// messages, and so on (see NSDraggingDestination protocol).
//
// NSMatrix (Additions)
// defines a method that returns the cell in a point, or nil

#include <AppKit/AppKit.h>

@protocol MatrixDraggingDelegate
- (NSDragOperation)matrix:(NSMatrix *)matrix
          draggingEntered:(id <NSDraggingInfo>)sender;
- (NSDragOperation)matrix:(NSMatrix *)matrix
          draggingUpdated:(id <NSDraggingInfo>)sender;
- (void)matrix:(NSMatrix *)matrix
    draggingExited:(id <NSDraggingInfo>)sender;
- (BOOL)matrix:(NSMatrix *)matrix
    prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)matrix:(NSMatrix *)matrix
    performDragOperation:(id <NSDraggingInfo>)sender;
- (void)matrix:(NSMatrix *)matrix
    concludeDragOperation:(id <NSDraggingInfo>)sender;
@end

@interface NSMatrix (NSDraggingDestination)
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender;
- (void)draggingExited:(id <NSDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender;
@end

@interface NSMatrix (Additions)
- (id)cellAtPoint:(NSPoint)p;
@end

#endif
