//
// SourceCodeReference
//
// Contains a reference to a source line in a file.
//

#include "SourceCodeReference.h"
#include "Macros.h"

@implementation SourceCodeReference

+ (SourceCodeReference *)referenceToFilename:(NSString *)name
                                 lineNumber:(int)line
{
    return [[[self alloc] initWithFilename:name lineNumber:line] autorelease];
}

- (id)initWithFilename:(NSString *)name
           lineNumber:(int)line
{
    self = [super init];
    if (self) {
        Assign(filename, name);
        lineNumber = line;
    }
    return self;
}

- (void)dealloc
{
    Assign(filename, nil);
    [super dealloc];
}

//
// accessors
//

- (NSString *)filename  { return filename; }
- (int)lineNumber       { return lineNumber; }


- (NSString *)description
{
    return [NSString stringWithFormat:@"%@:%d", filename, lineNumber];
}

//
// it can be hashed (NSObject protocol)
//
- (unsigned)hash
{
    return lineNumber;
}

- (BOOL)isEqual:(id)object
{
    if ((self == object)
        || ([object isKindOfClass:[SourceCodeReference class]]
            && lineNumber == [object lineNumber]
            && [filename isEqual:[object filename]]))
        return YES;
    return NO;
}

- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}


// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder
{
//    [super encodeWithCoder:coder];
    [coder encodeValuesOfObjCTypes:"@i", &filename, &lineNumber];
}

- (id)initWithCoder:(NSCoder *)coder
{
    self = [super init];//WithCoder:coder];
    [coder decodeValuesOfObjCTypes:"@i", &filename, &lineNumber];
    return self;
}
@end
