#include "UniqueString.h"
#include <Foundation/NSSet.h>
#include <Foundation/NSCoder.h>
#include <Foundation/NSInvocation.h>

@implementation UniqueString

NSMutableSet *TheUniqueStringsSet;

+ (void)initialize
{
    if (!TheUniqueStringsSet) {
#ifdef GNUSTEP
       TheUniqueStringsSet = [[NSMutableSet alloc] init];
#else
       TheUniqueStringsSet = [[NSMutableSet allocWithZone:NSCreateZone(0, NSPageSize(), NO)] init];
#endif
    }
}

+ (UniqueString *)stringWithString:(NSString *)s;
{
    UniqueString *uniquestring = [TheUniqueStringsSet member:s];
    if (!uniquestring) {
        uniquestring = s;//[[UniqueString allocWithZone:[TheUniqueStringsSet zone]] initWithString:s];
        [TheUniqueStringsSet addObject:uniquestring];
        //[uniquestring autorelease];
    }
    return uniquestring;
}

- (id)initWithString:(NSString *)s
{
//#ifndef GNUSTEP
    self = [super init];
//#endif
    if (self) {
        string = [s copyWithZone:[self zone]];
    }
    return self;
}

// primitive methods
//- (unsigned int)length;
//{
//    return [string length];
//}

#ifdef GNUSTEP
// GNUstep expects cStringLength to be defined
//- (unsigned int)cStringLength
//{
//    return [string cStringLength];
//}
#endif


//- (unichar)characterAtIndex:(unsigned)index;
//{
//    return [string characterAtIndex:index];
//}

int C1, C2, C3, C4;
+ (void)printCs {
    NSLog(@"%d %d %d %d", C1, C2, C3, C4);
}

- (NSString *)description
{
    return [string description];
}

// NSObject protocol
- (BOOL)isEqual:(id)object;
{
    C1++;
    if (self == object) return YES;
    C2++;
    if ([object class] == [UniqueString class]) return NO;
    C3++;
    return [string isEqual:object];
}

- (unsigned)hash;
{
    return [string hash];
}

// UniqueStrings are unique and are never released
- (id)retain { return self; }
- (oneway void)release { }
- (id)autorelease { return self; }
- (unsigned)retainCount { return 1; }

- (id)copyWithZone:(NSZone *)z { C4++; return self; }

// NSCoding Protocol
- (Class)classForCoder
{
    return [UniqueString class];
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:string];
}

- (id)initWithCoder:(NSCoder *)coder
{
    NSString *s;
    UniqueString *uniquestring;
    s = [coder decodeObject];
    uniquestring = [UniqueString stringWithString:s];
    [super release];
    return uniquestring;
}

//
// Forwards other messages to uniqued object
//

- (BOOL)respondsToSelector:(SEL)aSelector
{
    return [string respondsToSelector:aSelector];
}

- (void)forwardInvocation:(NSInvocation *)invocation
{
    [invocation invokeWithTarget:string];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)sel
{
    return [string methodSignatureForSelector:sel];
}

@end
