// DrawView+Drawing
// ----------------
// methods for drawing entities in DrawView.

#include "DrawView.h"
#ifdef GNUSTEP
#include <values.h>
#endif

extern int CiTR;
extern BOOL dontdraw;

@implementation DrawView (Drawing)

- (void)drawEventsWithDescriptor:(STEntityDescriptor *)entityDescriptor
                            rect:(NSRect)entityRect
                  fromEnumerator:(NSEnumerator *)enumerator
{
    pathfunction *path;
    drawfunction *draw;
    NSColor *color;
    NSColor *lastColor = nil;
    id <PajeEvent>entity;
    
    path = [entityDescriptor pathFunction];
    draw = [entityDescriptor drawFunction];
    PSgsave();

    while ((entity = [enumerator nextObject]) != nil) {
        NSRect r = entityRect;
        float x = TIMEtoX([filter timeForEntity:entity]);
        r.origin.x += x;

        color = [filter colorForEntity:entity];
        if (![color isEqual:lastColor]) {
            lastColor = color;
            [color set];
        }

        PSgsave();
        path(NSMinX(r), NSMinY(r), NSWidth(r), NSHeight(r));
        draw();
        PSgrestore();
    }
    PSgrestore();
}

#define drawBlackRect(x1, x2, r) \
do { \
    lastColor = [NSColor blackColor]; \
    [lastColor set]; \
    NSRectFill(NSMakeRect(x1, NSMinY(r), x2 - x1, NSHeight(r))); \
} while (0)

- (void)drawStatesWithDescriptor:(STEntityDescriptor *)entityDescriptor
                            rect:(NSRect)entityRect
                  fromEnumerator:(NSEnumerator *)enumerator
{
    pathfunction *path;
    drawfunction *draw;
    float insetBy;
    NSColor *color;
    NSColor *lastColor = nil;
    id <PajeState>entity;
    float firstUndrawnX = MAXFLOAT;
    float lastUndrawnX = MAXFLOAT;

    path = [entityDescriptor pathFunction];
    draw = [entityDescriptor drawFunction];
    insetBy = [entityDescriptor insetAmount];
    PSgsave();
    PSsetlinewidth(0.8);

    while ((entity = [enumerator nextObject]) != nil) {
        NSRect rect = entityRect;
        float x1 = TIMEtoX([filter startTimeForEntity:entity]);
        float x2 = TIMEtoX([filter endTimeForEntity:entity]);
        int imbricationLevel;
        float newHeight;
        rect.origin.x += x1;
        rect.size.width += (x2 - x1);
        imbricationLevel = [filter imbricationLevelForEntity:entity];
        newHeight = rect.size.height - insetBy * imbricationLevel;
        if (newHeight < 2)
            newHeight = 2;
        if (newHeight < rect.size.height)
            rect.size.height = newHeight;
        
        if (NSWidth(rect) < 2) {
#if 0
            if ((firstUndrawnX != MAXFLOAT) && (x1 > lastUndrawnX)) {
                drawBlackRect(firstUndrawnX, lastUndrawnX, rect);
                firstUndrawnX = x1;
            }
            if (firstUndrawnX == MAXFLOAT)
                firstUndrawnX = x1;
            lastUndrawnX = x2;
            continue;
#else
            if ((lastUndrawnX != MAXFLOAT) && (x2 < firstUndrawnX)) {
                drawBlackRect(firstUndrawnX, lastUndrawnX, rect);
                lastUndrawnX = x2;
            }
            if (lastUndrawnX == MAXFLOAT)
                lastUndrawnX = x2;
            firstUndrawnX = x1;
            continue;
#endif
        }

        if (firstUndrawnX != MAXFLOAT) {
            drawBlackRect(firstUndrawnX, lastUndrawnX, rect);
            firstUndrawnX = lastUndrawnX = MAXFLOAT;
        }

        color = [filter colorForEntity:entity];
        if (![color isEqual:lastColor]) {
            lastColor = color;
            [color set];
        }

#ifdef GNUSTEP
        // Very big rectangles are not drawn in GNUstep 
        rect = NSIntersectionRect(rect, cutRect);
#endif
        PSgsave();
        path(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
        draw();
        PSgrestore();
    }
    
    if (firstUndrawnX != MAXFLOAT) {
        drawBlackRect(firstUndrawnX, lastUndrawnX, entityRect);
    }
    PSgrestore();
}

- (void)drawValuesWithDescriptor:(STEntityDescriptor *)entityDescriptor
                            rect:(NSRect)entityRect
                  fromEnumerator:(NSEnumerator *)enumerator
{
    pathfunction *path;
    drawfunction *draw;
    NSColor *color;
    id <PajeState>entity;
    BOOL first = YES;
    float min;
    float max;
    float scale;
    float offset;
    PajeEntityType *entityType;
    NSColor *lastColor = nil;

    entityType = [entityDescriptor entityType];
    
    min = [[filter minValueForEntityType:entityType] floatValue];
    max = [[filter maxValueForEntityType:entityType] floatValue];

    if (min != max)
        scale = -NSHeight(entityRect) / (max - min);
    else
        scale = 1;
    offset = NSMaxY(entityRect) - min * scale;

    path = [entityDescriptor pathFunction];
    draw = [entityDescriptor drawFunction];
    color = [filter colorForEntityType:entityType];
    [color set];

    while ((entity = [enumerator nextObject]) != nil) {
        float x1 = TIMEtoX([filter startTimeForEntity:entity]);
        float x2 = TIMEtoX([filter endTimeForEntity:entity]);
        float y = [[filter valueForEntity:entity] doubleValue] * scale + offset;
        if (first) {
            first = NO;
            PSgsave();
            PSsetlinewidth(NSWidth(entityRect));
            PSmoveto(x1, y);
        }

#ifdef GNUSTEP
        // Very big lines are not drawn in GNUstep 
        if (x1 < NSMinX(cutRect)) x1 = NSMinX(cutRect);
        if (x2 > NSMaxX(cutRect)) x2 = NSMaxX(cutRect);
#endif
        PSlineto(x1, y);
        PSlineto(x2, y);

        color = [filter colorForEntity:entity];
        if (![color isEqual:lastColor]) {
            if (lastColor != nil) {
                PSstroke();
                PSmoveto(x2, y);
            }
            lastColor = color;
            [color set];
        }

    }
    if (!first) {
        PSstroke();
        PSgrestore();
    }
}


- (void)drawLinksWithDescriptor:(STEntityDescriptor *)entityDescriptor
                           rect:(NSRect)entityRect
                 fromEnumerator:(NSEnumerator *)enumerator
{
    pathfunction *path;
    drawfunction *draw;
    NSColor *color;
    NSColor *lastColor = nil;
    id <PajeLink>entity;

    path = [entityDescriptor pathFunction];
    draw = [entityDescriptor drawFunction];
    PSgsave();
    PSsetlinewidth(NSWidth(entityRect));

    while ((entity = [enumerator nextObject]) != nil) {
        NSRect rect1;
        NSRect rect2;
        float x1, x2, y1, y2;
        //NSRect r = entityRect;
        PajeContainer *sourceContainer;
        PajeContainer *destContainer;
        sourceContainer = [filter sourceContainerForEntity:entity];
        destContainer = [filter destContainerForEntity:entity];
        if (sourceContainer != nil) {
#if 0
            rect1 = [self rectForInstance:sourceContainer
                             ofEntityType:[filter sourceEntityTypeForEntity:entity]];
#else
            rect1 = [[self descriptorForEntityType:[sourceContainer entityType]] rectForInstance:sourceContainer];
#endif
            x1 = TIMEtoX([filter startTimeForEntity:entity]);
            y1 = NSMidY(rect1);
        }
        if (destContainer != nil) {
#if 0
            rect2 = [self rectForInstance:destContainer
                             ofEntityType:[filter destEntityTypeForEntity:entity]];
#else
            rect2 = [[self descriptorForEntityType:[destContainer entityType]] rectForInstance:destContainer];
#endif
            x2 = TIMEtoX([filter endTimeForEntity:entity]);
            y2 = NSMidY(rect2);
        }
        /*
        r.origin.x += x1;
        r.size.width += (x2 - x1);
        r.origin.y += y1;
        r.size.height += (y2 - y1);
         */

//        CiTR++;

//        if (!dontdraw) {
        color = [filter colorForEntity:entity];
        if (![color isEqual:lastColor]) {
            lastColor = color;
            [color set];
        }

        if (sourceContainer && destContainer) {
        PSgsave();
        //path(NSMinX(r), NSMinY(r), NSWidth(r), NSHeight(r));
        if (sourceContainer == nil) {
            pathfunction PSIn;
            PSIn(x1, y1, 8, 8);
        } else if (destContainer == nil) {
            pathfunction PSOut;
            PSOut(x1, y1, 8, 8);
        } else {
            path(x1, y1, x2-x1, y2-y1);
        }
        draw();
        PSgrestore();
        }
//        } // dontdraw
    }
    PSgrestore();
}

- (void)drawEntityType:(PajeEntityType *)entityType
           inContainer:(PajeContainer *)container
            insideRect:(NSRect)drawRect
{
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];
    id drawStartTime;
    id drawEndTime;
    NSEnumerator *enumerator;
    NSRect rect = [edesc rectInContainer:container];
    PajeDrawingType drawingType = [edesc drawingType];

    if (![edesc canDisplay]
        /* || ![filter existEntityType:entityType
                          inContainer:container]*/)
        return;

    if (drawingType != PajeLinkDrawingType) {
        NSRect r = rect;
        r.origin.x = drawRect.origin.x;
        r.size.width = drawRect.size.width;
        if (NSIsEmptyRect(NSIntersectionRect(r, drawRect)))
            return;
    }

    drawStartTime = XtoTIME(NSMinX(drawRect) + NSMinX(rect));
    drawEndTime   = XtoTIME(NSMaxX(drawRect) + NSMaxX(rect));
    enumerator = [filter enumeratorOfEntitiesTyped:entityType
                                       inContainer:container
                                          fromTime:drawStartTime
                                            toTime:drawEndTime];

    if (drawingType != PajeVariableDrawingType)
        enumerator = [[enumerator allObjects] reverseObjectEnumerator];

    switch (drawingType) {
        case PajeEventDrawingType:
            [self drawEventsWithDescriptor:edesc rect:rect
                            fromEnumerator:enumerator];
            break;
        case PajeStateDrawingType:
            [self drawStatesWithDescriptor:edesc rect:rect
                            fromEnumerator:enumerator];
            break;
        case PajeLinkDrawingType:
            [self drawLinksWithDescriptor:edesc rect:rect
                           fromEnumerator:enumerator];
            break;
        case PajeVariableDrawingType:
            [self drawValuesWithDescriptor:edesc rect:rect
                            fromEnumerator:enumerator];
            break;
        case PajeContainerDrawingType:
            break;
        default:
            NSAssert1(0, @"Invalid drawing type %d", drawingType);
    }

}

- (void)drawInstance:(id)entity
        ofEntityType:(PajeEntityType *)entityType
              inRect:(NSRect)drawRect
{
    NSEnumerator *subtypeEnum;
    PajeEntityType *subtype;
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];

    if (![edesc isInstance:entity inRect:drawRect])
        return;
        
    if ([[filter selectedContainers] containsObject:entity]) {
        [[NSColor whiteColor] set];
        NSRectFill([edesc rectForInstance:entity]);
    }

    //botei ali embaixo
    //[self drawEntityType:entityType
    //         inContainer:entity
    //          insideRect:drawRect];

/*    if (0 && ![hierarchyTree isLeaf]) {
        NSRect rect = [edesc rectForInstance:entity];
        int level = [hierarchyTree treeLevel];
        NSColor *c = [NSColor colorWithCalibratedWhite:(float)level/5+.25 alpha:1];
        [c set];
        NSRectFill(rect);
    }
*/

    // draw all instances of each subhierarchy
    subtypeEnum = [[filter containedTypesForContainerType:entityType] objectEnumerator];
    while ((subtype = [subtypeEnum nextObject]) != nil) {
        //if (![subtree isLeaf])
        [self drawAllInstancesOfEntityType:subtype
                               inContainer:entity
                                    inRect:drawRect];
    }
    /*
    subtypeEnum = [[filter containedTypesForContainerType:entityType] objectEnumerator];
    while ((subtype = [subtypeEnum nextObject]) != nil) {
        //if ([subtree isLeaf])
        [self drawAllInstancesOfEntityType:subtype
                               inContainer:entity
                                    inRect:drawRect];
    }
    */

    //aqui
    [self drawEntityType:entityType
             inContainer:entity
              insideRect:drawRect];
}

- (void)drawAllInstancesOfEntityType:(PajeEntityType *)entityType
                         inContainer:(PajeContainer *)container
                              inRect:(NSRect)drawRect
{
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];
    if (![edesc isContainer]) {
        [self drawEntityType:entityType
                 inContainer:container
                  insideRect:drawRect];
    } else {
        if (!container || [edesc isContainer:container inRect:drawRect]) {
            NSEnumerator *ienum;
            id instance;
                // check all instances on this hierarchy
            ienum = [filter enumeratorOfContainersTyped:entityType
                                            inContainer:container];
            while ((instance = [ienum nextObject]) != nil) {
                NSAutoreleasePool *pool = [NSAutoreleasePool new];
                [self drawInstance:instance
                      ofEntityType:entityType
                            inRect:drawRect];
                [pool release];
            }
        }
    }
}



@end
