// DrawView+Positioning
// --------------
// methods for finding positions of entities in DrawView.

#include "DrawView.h"

@implementation DrawView (Positioning)

- (void)setRect:(NSRect)rect
  forEntityType:(PajeEntityType *)type
    inContainer:(PajeContainer *)container
{
    STEntityDescriptor *entityDescriptor;

    entityDescriptor = [self descriptorForEntityType:type];

    if (entityDescriptor) {
        [entityDescriptor setRect:rect forContainer:container];
    }
}

- (NSRect)rectForEntityType:(PajeEntityType *)type
                inContainer:(PajeContainer *)container
{
    STEntityDescriptor *entityDescriptor;

    entityDescriptor = [self descriptorForEntityType:type];

    if (entityDescriptor) {
        return [entityDescriptor rectInContainer:container];
    }
    return NSZeroRect;
}

- (NSRect)rectForInstance:(id)entity
             ofEntityType:(PajeEntityType *)type
{
    STEntityDescriptor *entityDescriptor;

    entityDescriptor = [self descriptorForEntityType:type];

    if (entityDescriptor) {
        return [entityDescriptor rectForInstance:entity];
    }
    return NSZeroRect;
}

- (NSRect)calcRectForEntityType:(PajeEntityType *)entityType
                    inContainer:(PajeContainer *)container
                           minY:(float)minY
{
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];
    NSRect rect = [edesc drawingRect];
    if ([edesc drawingType] != PajeLinkDrawingType) {
        if ([edesc canDisplay]
            /* && [filter existEntityType:entityType
                              inContainer:container]*/) {
            rect.origin.y += minY;
        } else {
            rect.origin.y = minY;
            rect.size.height = 0;
        }
    }
    [self setRect:rect forEntityType:entityType inContainer:container];
    return rect;
}

- (NSRect)calcRectForInstance:(id)entity
                 ofEntityType:(PajeEntityType *)entityType
                         minY:(float)minY
{
    NSEnumerator *subtypeEnum;
    PajeEntityType *subtype;
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];
    NSRect rect = NSZeroRect;

    NSAssert([edesc isContainer], @"hierarchyTree shouldn't be leaf");

    subtypeEnum = [[filter containedTypesForContainerType:entityType] objectEnumerator];
    while ((subtype = [subtypeEnum nextObject]) != nil) {
        NSRect r;
        r = [self calcRectForAllInstancesOfEntityType:subtype
                                          inContainer:entity
                                                 minY:minY];
        if (!NSIsEmptyRect(r)) {
            rect = NSUnionRect(rect, r);
            minY = NSMaxY(rect);
        }
    }
    [edesc setRect:rect forInstance:entity];
    return rect;
}

- (NSRect)calcRectForAllInstancesOfEntityType:(PajeContainerType *)entityType
                                  inContainer:(PajeContainer *)container
                                         minY:(float)minY
{
    NSRect rect = NSZeroRect;
    float containerStartX;
    float containerEndX;
    STEntityDescriptor *edesc = [self descriptorForEntityType:entityType];
    if (![edesc isContainer]) {
        rect = [self calcRectForEntityType:entityType
                               inContainer:container
                                      minY:minY];
        containerStartX = TIMEtoX([filter startTimeForEntity:container]);
        containerEndX = TIMEtoX([filter endTimeForEntity:container]);
        rect.origin.x += containerStartX;
        rect.size.width += containerEndX - containerStartX;
        /*
        if ([container respondsToSelector:@selector(startTime)]) {
            rect.origin.x += TIMEtoX([container startTime]);
            rect.size.width += TIMEtoX([endTime earlierDate:[container endTime]]) - TIMEtoX([container startTime]);
        } else {
            rect.origin.x += TIMEtoX(startTime);
            rect.size.width += TIMEtoX(endTime) - TIMEtoX(startTime);
        }
        */
    } else {
        NSEnumerator *ienum;
        id instance;

        rect.origin.y = minY;
        rect.size.height = [edesc offset];
        containerStartX = TIMEtoX([filter startTimeForEntity:container]);
        containerEndX = TIMEtoX([filter endTimeForEntity:container]);
        rect.origin.x += containerStartX;
        rect.size.width += containerEndX - containerStartX;
/*        if ([container respondsToSelector:@selector(startTime)]) {
            rect.origin.x = TIMEtoX([startTime laterDate:[container startTime]]);
            rect.size.width = TIMEtoX([endTime earlierDate:[container endTime]]) - TIMEtoX([startTime laterDate:[container startTime]]);
        }*/
        minY = NSMaxY(rect);
        
        // check all instances on this hierarchy
        ienum = [filter enumeratorOfContainersTyped:entityType
                                        inContainer:container];
        while ((instance = [ienum nextObject]) != nil) {
            NSRect r;
            r = [self calcRectForInstance:instance
                             ofEntityType:entityType
                                     minY:minY];
            if (!NSIsEmptyRect(r)) {
                rect = NSUnionRect(rect, r);
                rect.size.height += [edesc separation];
                minY = NSMaxY(rect);
            }
        }
        if (container)
            [edesc setRect:rect forContainer:container];
    }
    return rect;
}

@end
