/*
 * DrawView.m
 *
 * a View that draws lines
 *
 * 19960115 BS  creation
 * 19960212 BS  clean-up
 */

#include "DrawView.h"
#ifdef GNUSTEP
#include <AppKit/PSOperators.h>
#endif
#ifdef __APPLE__
#include "PSOperators.h"
#endif
#if defined(NEXT)
#import <AppKit/psopsOpenStep.h>	// For PS and DPS function prototypes
#import "drawline.h"
#endif
#include "../General.bproj/NSUserDefaults+Colors.h"
#include "EntityTypeInspector.h"
#include "../General.bproj/Macros.h"

#if defined(GNUSTEP) || defined(__APPLE__)
void PSInit(void){}
#endif

@implementation DrawView

- (void)dealloc
{
    [entityDescriptors release];
    [filter release];
    [startTime release];
    [endTime release];
    [backgroundColor release];
    [selectedBackgroundColor release];
    [super dealloc];
}


- (NSColor *)backgroundColor
{
    return backgroundColor;
}

- (NSColor *)selectedBackgroundColor
{
    return selectedBackgroundColor;
}

- (void)setBackgroundColor:(NSColor *)color
{
    Assign(backgroundColor, color);
    [[NSUserDefaults standardUserDefaults]
              setColor:backgroundColor
                forKey:[NSStringFromClass([self class]) stringByAppendingString:@"BackgroundColor"]];
}

- (void)setSelectedBackgroundColor:(NSColor *)color
{
    Assign(selectedBackgroundColor, color);
    [[NSUserDefaults standardUserDefaults]
              setColor:selectedBackgroundColor
                forKey:[NSStringFromClass([self class]) stringByAppendingString:@"SelectedBackgroundColor"]];
}

- (void)setFilter:(PajeFilter *)newFilter
{
    Assign(filter, newFilter);
}

- (PajeFilter *)filter
{
    return filter;
}

- (void)hierarchyChanged
{
    Assign(startTime, [filter startTime]);
    Assign(endTime, [filter endTime]);

    if (startTime != nil && endTime != nil) {
        [self adjustSize];
    }
}

- (void)dataChangedForEntityType:(PajeEntityType *)entityType
{
    if ([[entityDescriptors member:entityType] canDisplay]) {
        [self adjustSize];
    }
}

- (void)colorChangedForEntityType:(PajeEntityType *)entityType
{
    if ([[entityDescriptors member:entityType] canDisplay]) {
        [self setNeedsDisplay:YES];
    }
}

- (void)orderChangedForContainerType:(PajeContainerType *)containerType
{
    [self adjustSize];
}

- (void)awakeFromNib
{
    NSCursor *cursor;
    NSImage *cursorImage;
    NSString *cursorPath;
    
    // initialize postscript functions.
    PSInit();

    // initialize instance variables

    entityDescriptors = [[NSMutableSet alloc] init];

    pointsPerSecond = 10000;
    hasZoomed = NO;
    filter = nil;
    trackingRectTag = 0;
    timeUnitDivisor = 1;
    [timeUnitField setStringValue:@"s"];
    [cursorTimeField setFloatingPointFormat:NO left:6 right:6];

//teste    [displaySelectionMatrix renewRows:2 columns:0];

#ifndef GNUSTEP
    if (0) {
        int i;
        for (i=0; i<5; i++)
            [[zoomButtonsMatrix cellWithTag:i] setGradientType:NSGradientConcaveStrong];
    }
#endif
    
    [zoomToSelectionButton setEnabled:NO];

//    [[self window] setBackingType:NSBackingStoreNonretained];

    // get background color from defaults
    Assign(backgroundColor, [[NSUserDefaults standardUserDefaults]
                                colorForKey:[NSStringFromClass([self class])
                                   stringByAppendingString:@"BackgroundColor"]]);
    if (backgroundColor == nil) {
        Assign(backgroundColor, [NSColor controlBackgroundColor]);
    }
    Assign(selectedBackgroundColor, [[NSUserDefaults standardUserDefaults]
                                colorForKey:[NSStringFromClass([self class])
                                   stringByAppendingString:@"SelectedBackgroundColor"]]);
    if (selectedBackgroundColor == nil) {
        Assign(selectedBackgroundColor, [NSColor controlLightHighlightColor]);
    }

    // initialize the cursor
    cursorPath = [[NSBundle bundleForClass:[self class]]
                     pathForImageResource:@"crosscursor"];
    cursorImage = [[NSImage alloc] initWithContentsOfFile:cursorPath];
    cursor = [[NSCursor alloc] initWithImage:cursorImage hotSpot:NSMakePoint(7,9)];
    [cursorImage release];
    if (cursor != nil) {
        [[self enclosingScrollView] setDocumentCursor:cursor];
        [cursor release];
    }

#ifdef GNUSTEP
[[cursorTimeField window] performSelector:@selector(makeKeyAndOrderFront:) withObject:self afterDelay:0];
#endif

    // register to receive color drag-and-drops
    [self registerForDraggedTypes:[NSArray arrayWithObject:NSColorPboardType]];

    [[EntityTypeInspector alloc] initWithDelegate:self];

    [[self enclosingScrollView] setHasHorizontalRuler:YES];
    [[[self enclosingScrollView] horizontalRulerView] setReservedThicknessForMarkers:0.0];
    [[self enclosingScrollView] setBackgroundColor:[self backgroundColor]/*[NSColor whiteColor]*/];
    [[self enclosingScrollView] setDrawsBackground:YES/*NO*/];

    //TODO: should register as tool
}

- (void)setupDisplaySelectionMatrix
{
    int rowsPerColumn = 2;
    int cellCount;
    int cellNo;
    NSEnumerator *edescEnum;
    STEntityDescriptor *edesc;
    int column = 0, row = rowsPerColumn;
    NSButtonCell *cell;
    NSMutableArray *types = [NSMutableArray array];
    
    edescEnum = [entityDescriptors objectEnumerator];
    while ((edesc = [edescEnum nextObject]) != nil) {
        if (![edesc isContainer] && ([edesc entityType] != nil)) {
            [types addObject:[edesc entityType]];
        }
    }

    cellCount = [types count];
    
    if ((cellCount % rowsPerColumn) == 0) {
        [displaySelectionMatrix renewRows:rowsPerColumn
                                  columns:(cellCount / rowsPerColumn)];
    } else {
        [displaySelectionMatrix renewRows:rowsPerColumn
                                  columns:(cellCount / rowsPerColumn) + 1];
    }
    
    for (cellNo = 0; cellNo < cellCount; cellNo++) {
        id entityType = [types objectAtIndex:cellNo];
        STEntityDescriptor *entityDescriptor;

        entityDescriptor = [entityDescriptors member:entityType];
        column = cellNo / rowsPerColumn;
        row = cellNo % rowsPerColumn;
        cell = [displaySelectionMatrix cellAtRow:row column:column];
        [cell setTitle:[entityDescriptor description]];
        [cell setRepresentedObject:entityDescriptor];
        [cell setState:[entityDescriptor canDisplay]];
        [cell setTransparent:NO];
        [cell setEnabled:YES];
        [cell setWraps:NO];
    }

    for (row++; row < rowsPerColumn; row++) {
        cell = [displaySelectionMatrix cellAtRow:row column:column];
        [cell setTransparent:YES];
        [cell setEnabled:NO];
    }

    [displaySelectionMatrix setIntercellSpacing:NSMakeSize(0,-2)];
    [displaySelectionMatrix sizeToFit];
    [displaySelectionMatrix setNeedsDisplay:YES];
    [[displaySelectionMatrix superview] setNeedsDisplay:YES];//teste
}

- (IBAction)displaySelectionChanged:sender;
{
    NSCell *cell = [sender selectedCell];
    STEntityDescriptor *entityDescriptor = [cell representedObject];
    //[[cell representedObject] setCanDisplay:[cell state]];
    [entityDescriptor setCanDisplay:![filter isHiddenEntityType:[entityDescriptor entityType]]];
    [self redisplay:self];
}


- (NSSet *)entityDescriptors { return entityDescriptors; }

- (STEntityDescriptor *)descriptorForEntityType:(PajeEntityType *)entityType
{
    STEntityDescriptor *edesc = [entityDescriptors member:entityType];

    if (!edesc) {
        edesc = [STEntityDescriptor descriptorWithEntityType:entityType
                                                 drawingType:[filter drawingTypeForEntityType:entityType]];
        [entityDescriptors addObject:edesc];
        [self setupDisplaySelectionMatrix];
    }
    return edesc;
}


- (NSRect)adjustScroll:(NSRect)newVisible
{
    if (hasZoomed) {
        float lastX = TIMEtoX(endTime);
        float newX = TIMEtoX(oldMiddleTime) - NSWidth(newVisible) / 2;
        if ( (newX + NSWidth(newVisible)) > lastX )
            newX = lastX - NSWidth(newVisible);
        if (newX < 0) newX = 0;
        newVisible.origin.x = newX;
        hasZoomed = NO;
        [oldMiddleTime release];
        oldMiddleTime = nil;
    }
    return newVisible;
}

- (void)redisplay:sender
{
    [self adjustSize];
    [self setNeedsDisplay:YES];
}

- (void)windowDidResize:(NSNotification *)aNotification
{
    [self adjustSize];
}

- (BOOL)shouldDisplayEntityType:(PajeEntityType *)entityType
                    inContainer:(PajeContainer *)container;
{
return ![filter isHiddenEntityType:entityType];
//    if (![[self descriptorForEntityType:entityType] canDisplay])
//        return NO;
//    return YES;
}

- (id)rootInstance
{
    return [filter rootInstance];
}

- (void)setRulerUnit
{
    NSRulerView *ruler = [[self enclosingScrollView] horizontalRulerView];

    if (ruler) {
        // sets ruler scale
        NSArray *upArray;
        NSArray *downArray;
        unichar microsecond[] = {0x00b5, 's'};

        upArray = [NSArray arrayWithObjects:
            [NSNumber numberWithFloat:5.0], [NSNumber numberWithFloat:2.0], nil];
        downArray = [NSArray arrayWithObjects:
            [NSNumber numberWithFloat:0.5], [NSNumber numberWithFloat:0.2], nil];
        if (pointsPerSecond > 300000000) {
            timeUnitDivisor = 1000000000;
            [cursorTimeField setFloatingPointFormat:NO left:12 right:0];
            [timeUnitField setStringValue:@"ns"];
            [NSRulerView registerUnitWithName:@"nanoseconds"
                                 abbreviation:@"ns"
                 unitToPointsConversionFactor:pointsPerSecond/1000000000.0
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"nanoseconds"];
        } else if (pointsPerSecond > 300000) {
            timeUnitDivisor = 1000000;
            [cursorTimeField setFloatingPointFormat:NO left:12 right:0];
            [timeUnitField setStringValue:[NSString stringWithCharacters:microsecond length:2]]; //@"us";
            [NSRulerView registerUnitWithName:@"microseconds"
                                 abbreviation:@"us"
                 unitToPointsConversionFactor:pointsPerSecond/1000000.0
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"microseconds"];
        } else if (pointsPerSecond > 300) {
            timeUnitDivisor = 1000;
            [cursorTimeField setFloatingPointFormat:NO left:9 right:3];

            [timeUnitField setStringValue:@"ms"];
            [NSRulerView registerUnitWithName:@"milliseconds"
                                 abbreviation:@"ms"
                 unitToPointsConversionFactor:pointsPerSecond/1000.0
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"milliseconds"];
        } else if (pointsPerSecond > 0.1) {
            timeUnitDivisor = 1;
            [cursorTimeField setFloatingPointFormat:NO left:6 right:6];
            [timeUnitField setStringValue:@"s"];
            [NSRulerView registerUnitWithName:@"seconds"
                                 abbreviation:@"s"
                 unitToPointsConversionFactor:pointsPerSecond
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"seconds"];
        } else if (pointsPerSecond > .001) {
            timeUnitDivisor = 1.0/3600.0;
            [cursorTimeField setFloatingPointFormat:NO left:6 right:6];
            [timeUnitField setStringValue:@"h"];
            [NSRulerView registerUnitWithName:@"hours"
                                 abbreviation:@"h"
                 unitToPointsConversionFactor:pointsPerSecond*3600.0
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"hours"];
        } else {
            timeUnitDivisor = 1.0/3600.0/24.0;
            [cursorTimeField setFloatingPointFormat:NO left:6 right:6];
            [timeUnitField setStringValue:@"d"];
            [NSRulerView registerUnitWithName:@"days"
                                 abbreviation:@"d"
                 unitToPointsConversionFactor:pointsPerSecond*3600.0*24.0
                                  stepUpCycle:upArray
                                stepDownCycle:downArray];
            [ruler setMeasurementUnits: @"days"];
        }
    }
}

- (void)adjustSize
{
    NSRect newBounds;
    NSScrollView *scrollView = [self enclosingScrollView];
    NSRulerView *ruler = [scrollView horizontalRulerView];
    id rootInstance = [self rootInstance];

    [self setRulerUnit];

    // calculate new sizes
    [entityDescriptors makeObjectsPerformSelector:@selector(reset)];
    
    [self performSelector:@selector(verifyTimes:)
               withObject:self
               afterDelay:0.0];

    if (rootInstance) {
        float delta;
        newBounds = [self calcRectForInstance:rootInstance
                                 ofEntityType:[filter entityTypeForEntity:rootInstance]
                                         minY:0];
        delta = TIMEtoX(startTime) - NSMinX(newBounds);
        if (delta > 0) {
            newBounds.origin.x += delta;
            newBounds.size.width -= delta;
        }
        delta = NSMaxX(newBounds) - TIMEtoX(endTime);
        if (delta > 0) {
            newBounds.size.width -= delta;
        }

        newBounds.size.height += 1;
        [self setBoundsOrigin:newBounds.origin];
        [self setFrameSize:newBounds.size];

        [self setupDisplaySelectionMatrix];

        [ruler setOriginOffset:-[self bounds].origin.x];
#ifdef GNUSTEP
//	[ruler invalidateHashMarks];
#endif
        [scrollView tile];
        [scrollView setNeedsDisplay:YES];
        
    // change mouse tracking rect
        if (trackingRectTag) [self removeTrackingRect:trackingRectTag];
        trackingRectTag = [self addTrackingRect:[self visibleRect] owner:self
                                       userData:NULL assumeInside:NO];
//    [[self window] setAcceptsMouseMovedEvents:YES];
    [self setNeedsDisplay:YES];//[[[self window] contentView] setNeedsDisplay:YES];
    }
}

- (void)verifyTimes:sender
{
    NSScrollView *scrollView = [self enclosingScrollView];
    [filter verifyStartTime:startTime
                    endTime:XtoTIME(NSMaxX([self convertRect:[scrollView frame] fromView:scrollView]))];
}

/*
 * Changing scales
 * ------------------------------------------------------------------------
 */

- (void)incrementThreadScale:sender
{
}

- (void)decrementThreadScale:sender
{
}

- (void)incrementThreadDistance:sender
{
}

- (void)decrementThreadDistance:sender
{
}

- (void)doubleTimeScale:sender
{
    NSRect visible;

    visible = [self visibleRect];
    oldMiddleTime = [XtoTIME(NSMidX(visible)) retain];
    pointsPerSecond *= 2;
    hasZoomed = YES;
    [self adjustSize];

    if (pointsPerSecond > 2e6) {
        [doubleTimeScaleButton setEnabled:NO];
    }
}

- (void)halveTimeScale:sender
{
    NSRect visible;

    visible = [self visibleRect];
    oldMiddleTime = [XtoTIME(NSMidX(visible)) retain];
    pointsPerSecond /= 2;
//    if ( TIMEtoX(endTime - startTime) < NSWidth(visible) )
//        pointsPerSecond = NSWidth(visible) / (endTime - startTime);
    hasZoomed = YES;
    [self adjustSize];

    [doubleTimeScaleButton setEnabled:YES];
}

- (void)zoomToSelection:sender
{
    NSRect visible;
    double start, end;

    if (!selectionExists)
        return;

    start = [selectionStartTime timeIntervalSinceReferenceDate];
    end = [selectionEndTime timeIntervalSinceReferenceDate];
    visible = [[self superview] frame];
    oldMiddleTime = [XtoTIME((TIMEtoX(selectionStartTime) + TIMEtoX(selectionEndTime)) / 2) retain];
    pointsPerSecond = NSWidth(visible)
                    / [selectionEndTime timeIntervalSinceDate:selectionStartTime];
    if (pointsPerSecond > 2e6) {
        pointsPerSecond = 2e6;
        [doubleTimeScaleButton setEnabled:NO];
    } else {
        [doubleTimeScaleButton setEnabled:YES];
    }
    hasZoomed = YES;
    [self adjustSize];
}

- (float)pointsPerSecond
{
    return pointsPerSecond;
}

- (NSDate *)startTime
{
    return startTime;
}



/*
 * Drawing
 * ------------------------------------------------------------------------
 */

- (BOOL)isFlipped
// y-axis is inverted (y-coords augment from top to bottom)
{
    return YES;
}

- (BOOL)isOpaque
// this view draws every pixel it owns
{
    return YES;
}


- (NSRect)rectForEntity:(id<PajeEntity>)entity
{
    float x, x1, x2, y1, y2, min, max, scale, offset;
    NSRect rect, rect1, rect2;
    id entityType;
    id <PajeLink> link = entity;
    int imbricationLevel;
    float newHeight;

    if (entity == nil) return NSZeroRect;

    entityType = [filter entityTypeForEntity:entity];
    switch ([filter drawingTypeForEntity:entity]) {
        case PajeEventDrawingType:
            rect = [self rectForEntityType:entityType
                               inContainer:[filter containerForEntity:entity]];
            x = TIMEtoX([filter timeForEntity:entity]);
            rect.origin.x += x;
            break;
        case PajeStateDrawingType:
            rect = [self rectForEntityType:entityType
                               inContainer:[filter containerForEntity:entity]];
            x1 = TIMEtoX([filter startTimeForEntity:entity]);
            x2 = TIMEtoX([filter endTimeForEntity:entity]);
            rect.origin.x += x1;
            rect.size.width += (x2 - x1);
            imbricationLevel = [filter imbricationLevelForEntity:entity];
            newHeight = rect.size.height - [[self descriptorForEntityType:entityType] insetAmount] * imbricationLevel;
            if (newHeight < 2)
                newHeight = 2;
            if (newHeight < rect.size.height)
                rect.size.height = newHeight;
            break;
        case PajeVariableDrawingType:
            rect = [self rectForEntityType:entityType
                               inContainer:[filter containerForEntity:entity]];
            x1 = TIMEtoX([filter startTimeForEntity:entity]);
            x2 = TIMEtoX([filter endTimeForEntity:entity]);
            min = [[filter minValueForEntityType:entityType] floatValue];
            max = [[filter maxValueForEntityType:entityType] floatValue];

            if (min != max)
                scale = -NSHeight(rect) / (max - min);
            else
                scale = 1;
            offset = NSMaxY(rect) - min * scale;
            y1 = [[filter valueForEntity:entity] doubleValue] * scale + offset;
            y2 = NSMaxY(rect);
            rect.origin.x = x1;
            rect.size.width = (x2 - x1);
            rect.origin.y = y1;
            rect.size.height = y2 - y1;
            break;
        case PajeLinkDrawingType:
            //rect = [self rectForEntityType:entityType
            //                   inContainer:[filter containerForEntity:link]];
#if 1
            rect1 = [self rectForInstance:[filter sourceContainerForEntity:link]
                             ofEntityType:[filter sourceEntityTypeForEntity:link]];
            rect2 = [self rectForInstance:[filter destContainerForEntity:link]
                             ofEntityType:[filter destEntityTypeForEntity:link]];
#else
            {
                id sc = [filter sourceContainerForEntity:link];
                id dc = [filter destContainerForEntity:link];
                rect1 = [[self descriptorForEntityType:[sc entityType]] rectForInstance:sc];
                rect2 = [[self descriptorForEntityType:[dc entityType]] rectForInstance:dc];
            }
#endif
            x1 = TIMEtoX([filter startTimeForEntity:entity]);
            x2 = TIMEtoX([filter endTimeForEntity:entity]);
            y1 = NSMidY(rect1);
            y2 = NSMidY(rect2);
            rect.origin.x = x1;
            rect.size.width = (x2 - x1);
            rect.origin.y = y1;
            rect.size.height = (y2 - y1);
            break;
        default:
            rect = NSZeroRect;
    }

    return rect;
}

- (void)highlightEntity:(PajeEntity *)entity
{
    NSColor *color;
    NSRect rect;
    STEntityDescriptor *entityDescriptor;
    pathfunction *path;
    drawfunction *highlight;

    rect = [self rectForEntity:entity];

    if (NSEqualRects(rect, NSZeroRect))
        return;
    
    color = [filter colorForEntity:entity];
    color = [color highlightWithLevel:.2];

    entityDescriptor = [self descriptorForEntityType:[filter entityTypeForEntity:entity]];

    path = [entityDescriptor pathFunction];
    highlight = [entityDescriptor highlightFunction];

    [color set];
#ifdef GNUSTEP
    // big rectangles are not drawn in GNUstep. cut them
    // BUG: only things displayed as rectangles should be cut this way. (links won't work).
    if ([entityDescriptor drawingType] == PajeStateDrawingType) {
        rect = NSIntersectionRect(rect, cutRect);
    }
#endif
    PSgsave();
    path(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
    highlight();
    PSgrestore();
}

- (void)highlightEntities:(NSArray *)entities
{
    id eEnum, entity;

    if (entities != nil) {
        [self lockFocus];
        
        eEnum = [entities objectEnumerator];
        while ((entity = [eEnum nextObject]) != nil) {
            if (1||([self shouldDisplayEntityType:[filter entityTypeForEntity:entity]
                                  inContainer:[filter containerForEntity:entity]]
                && [filter canHighlightEntity:entity])) {
                [self highlightEntity:entity];
            }
        }
        
        [self unlockFocus];
    }
}

- (void)setHighlightedEntity:(PajeEntity *)entity
// highlights entity (and related entities), taking care of unhighlighting previous
{
    if (entity) {
        if (highlightedEntity != entity) {
            if (highlightedEntity) {
                [highlightedEntity release];
                [[self window] restoreCachedImage];
            } else
                [[self window] cacheImageInRect:
                    [self convertRect:[self visibleRect] toView:nil]];
            highlightedEntity = [entity retain];

            [entityNameField setStringValue:[filter descriptionForEntity:highlightedEntity]];

            [self highlightEntities:[[filter relatedEntitiesForEntity:entity] arrayByAddingObject:highlightedEntity]];

            [[self window] flushWindowIfNeeded];
        }
    } else {
        if (highlightedEntity) {
            [highlightedEntity release];
            highlightedEntity = nil;
            [[self window] restoreCachedImage];
            [[self window] flushWindowIfNeeded];
        }
        [entityNameField setStringValue:@""];
    }
}


- (void)drawBackgroundInRect:(NSRect)rect
// draws the background
{
    // draw background
    [backgroundColor set];
    NSRectFill(rect);

    // draw selection background
    if (selectionExists) {
        float selectionStart = TIMEtoX(selectionStartTime);
        float selectionWidth = TIMEtoX(selectionEndTime) - selectionStart;
        NSRect selectionRect = NSMakeRect(selectionStart, NSMinY(rect),
                                          selectionWidth, NSMaxY(rect));
#ifdef GNUSTEP
        // GNUstep can't draw big rects.
        selectionRect = NSIntersectionRect(selectionRect, cutRect);
#endif

        [selectedBackgroundColor set];
        NSRectFill(selectionRect);
        
        /*
        [[NSColor darkGrayColor] set];
        PSmoveto(NSMinX(selectionRect), NSMinY(selectionRect));
        PSlineto(NSMinX(selectionRect), NSMaxY(selectionRect));
        PSmoveto(NSMaxX(selectionRect), NSMinY(selectionRect));
        PSlineto(NSMaxX(selectionRect), NSMaxY(selectionRect));
        PSstroke();
        */
    }
}

void PSStrokeLine(float x, float y, float w, float h);

int CiTR;
BOOL dontdraw;

- (void)drawRect:(NSRect)rect
{
    // draws some more pixels, for the graphs that expand their limits (events)
    id drawStartTime = XtoTIME(NSMinX(rect) - 5);
    id drawEndTime   = XtoTIME(NSMaxX(rect) + 5);

    if (!startTime) return;

    // KLUDGE to see if trace changes (something is read)
    [self setNeedsDisplay:NO];
    [filter verifyStartTime:drawStartTime endTime:drawEndTime];
    if ([self needsDisplay]) {
        [self display];
        return;
    }

#ifdef GNUSTEP
    // Big rectangles are not drawn by GNUstep. Cut them with cutRect.
    cutRect = NSInsetRect([self visibleRect], -2.0, -2.0);
#endif
    
    NS_DURING

        // set blackground
        [self drawBackgroundInRect:rect];
        
        PSsetlinewidth(1);

        {
/*            NSDate *d1=[NSDate date], *d2, *d3;
            double t0,t1,t;
            CiTR=0;dontdraw=YES;
            [self drawInstance:[self rootInstance]
               ofHierarchyTree:[filter hierarchyTree]
                        inRect:rect];
            d2 = [NSDate date];
            t0 = [d2 timeIntervalSinceDate:d1];
            CiTR=0;dontdraw=NO;
            */
            [self drawInstance:[self rootInstance]
                  ofEntityType:[[self rootInstance] entityType]
                        inRect:rect];
/*            d3 = [NSDate date];
            t1 = [d3 timeIntervalSinceDate:d2];
            t=t0-t1;
            NSLog(@"drawing %d ents:%f-%f=%f s -> %f s/e", CiTR, t1,t0,t, CiTR ? t/CiTR : 0);
*/
        }
    NS_HANDLER
        NSLog(@"Ignoring exception caught inside drawRect: %@", localException);
    NS_ENDHANDLER
}






/*
 * Selection control
 * ------------------------------------------------------------------------
 */

- (void)changeSelectionWithPoint:(NSPoint)point
{
    NSRect redisplayRect;
    NSDate *cursorTime;
    NSRect frameRect = [self frame];

//NSLog(@"sel-last:%@ pt=%@", NSStringFromPoint(selectionLastPoint), //NSStringFromPoint(point));
//NSLog(@"frame=%@", NSStringFromRect(frameRect));
    if (point.x < NSMinX(frameRect)) point.x = NSMinX(frameRect);
    if (point.x > NSMaxX(frameRect)) point.x = NSMaxX(frameRect);
//NSLog(@"pt=%@", NSStringFromPoint(point));

    if (point.x == selectionLastPoint.x)
        return;

    cursorTime = XtoTIME(point.x);
    //[cursorTimeField setDoubleValue:[cursorTime timeIntervalSinceDate:startTime] * timeUnitDivisor];
    [cursorTimeField setDoubleValue:[cursorTime timeIntervalSinceReferenceDate] * timeUnitDivisor];

    redisplayRect = [self visibleRect];
    if (point.x < selectionLastPoint.x) {
        redisplayRect.origin.x = point.x;
        redisplayRect.size.width = selectionLastPoint.x - redisplayRect.origin.x + 1;
    } else {
        redisplayRect.origin.x = selectionLastPoint.x - 2;
        redisplayRect.size.width = point.x - redisplayRect.origin.x + 1;
    }

    [selectionStartTime release];
    [selectionEndTime release];
    if (point.x < selectionAnchorPoint.x) {
        selectionStartTime = [cursorTime retain];
        selectionEndTime = [XtoTIME(selectionAnchorPoint.x) retain];
    } else {
        selectionEndTime = [cursorTime retain];
        selectionStartTime = [XtoTIME(selectionAnchorPoint.x) retain];
    }

    selectionLastPoint = point;

    [self setNeedsDisplayInRect:redisplayRect];
    [self scrollRectToVisible:redisplayRect];
}

- (void)setNeedsDisplayInSelection
{
    NSRect redisplayRect = [self visibleRect];
    redisplayRect.origin.x = TIMEtoX(selectionStartTime);
    redisplayRect.size.width = TIMEtoX(selectionEndTime) - redisplayRect.origin.x + 1;
    [self setNeedsDisplayInRect:redisplayRect];
}

- (void)selectAll:(id)sender
{
    if (selectionExists) {
        [selectionStartTime release];
        [selectionEndTime release];
    }
    selectionStartTime = [startTime retain];
    selectionEndTime = [endTime retain];
    selectionExists = YES;
    [[NSNotificationCenter defaultCenter]
              postNotificationName:@"PajeTimeSelectionChangedNotification"
                            object:[self rootInstance]
                          userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                              selectionStartTime, @"StartTime",
                              selectionEndTime, @"EndTime", nil]];
    [self setNeedsDisplay:YES];
}



/*
 * Dragging control (NSDraggingDestination protocol)
 * ------------------------------------------------------------------------
 */

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
    NSPoint point = [self convertPoint:[sender draggingLocation] fromView:nil];

    [self setHighlightedEntity:[self findEntityAtPoint:point]];
#ifdef GNUSTEP
return NSDragOperationCopy;
#else
//    if (highlightedEntity)
        return NSDragOperationAll;
//    else
//        return NSDragOperationNone;
#endif
}

#ifdef GNUSTEP
// this shouldn't be needed (should be the defaul behaviour of NSView)
- (BOOL) prepareForDragOperation: (id <NSDraggingInfo>)sender
{
    return YES;
}
#endif
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    NSColor *draggedColor;

    draggedColor = [NSColor colorFromPasteboard:[sender draggingPasteboard]];
    if (draggedColor == nil)
        return NO;

    if (highlightedEntity) {
        [filter setColor:draggedColor forEntity:highlightedEntity];
        [self setHighlightedEntity:nil];
    } else {
        //FIXME: should test if inside selection to setSelectedBackgroundcolor:
        [self setBackgroundColor:draggedColor];
   }
    [self setNeedsDisplay:YES];
    return YES;
}



/*
 * Printing
 * ------------------------------------------------------------------------
 */

- (void)endPrologue
/*
 * Spit out the custom PostScript defs.
 */
{
    PSInit();
    [super endPrologue];
}


#ifdef GNUSTEP
- (void)setCursorTimeField:(id)field { cursorTimeField = [field retain]; }
- (void)setEntityNameField:(id)field { entityNameField = [field retain]; }
//- (void)setSelStateButton:(id)button { selStateButton = button; }
//- (void)setSelEventButton:(id)button { selEventButton = button; }
//- (void)setSelCommButton:(id)button { selCommButton = button; }
#endif
@end
