#include "EntityTypeInspector.h"
#if defined(GNUSTEP_XGPS) || defined(NEXT)
#import "drawline.h" // for pswraps
#endif
#include "STEntityDescriptor.h"
#include "DrawView.h"
#ifdef GNUSTEP
#include <AppKit/PSOperators.h>
#endif
#ifdef __APPLE__
#import "PSOperators.h"
#endif

@interface MyCustomImageRep : NSCustomImageRep
{
    void *function;
}
- (void *)function;
- (void)setFunction:(void *)f;
@end
@implementation MyCustomImageRep
- (void *)function             { return function; }
- (void)setFunction:(void *)f
{
    NSAssert(f, @"Invalid image drawing function");
    function = f;
}
@end


@implementation EntityTypeInspector

- (id)initWithDelegate:(id)del
{
    self = [super init];
    if (self) {
        delegate = [del retain];
    }
    return self;
}

- (void)dealloc
{
    [delegate release];
    [super dealloc];
}

- (NSString *)toolName
{
    return @"Shapes & sizes";
}

- (void)activateTool:(id)sender
/* sent by PajeController when the user selects this Tool */
{
    if (!shapeMatrix) {
        if (![NSBundle loadNibNamed:@"EntityTypeInspector" owner:self]) {
            NSRunAlertPanel(@"EntityTypeInspector",
                            @"Couldn't load interface file",
                            nil, nil, nil);
            return;
        }
        [self setupPopUpWithEntityDescriptors:[delegate entityDescriptors]];
        [self setupShapeMatrix];
        [self setupDrawMatrixes];
    }
    
    [[shapeMatrix window] orderFront:self];
}

- (void)setupPopUpWithEntityDescriptors:(NSSet *)set
{
    NSEnumerator *enumerator = [set objectEnumerator];
    STEntityDescriptor *item;
    
    [entityTypePopUp removeAllItems];

    while ((item = [enumerator nextObject]) != nil) {
        [entityTypePopUp addItemWithTitle:[item description]];
        [[entityTypePopUp lastItem] setRepresentedObject:item];
    }

    [entityTypePopUp selectItemAtIndex:0];
}

- (void)setupShapeMatrix
{
    MyCustomImageRep *imageRep;
    NSImage *image;
    NSSize size = [shapeMatrix cellSize];
    int col = 0;
    NSEnumerator *shapeFunctionsValueEnumerator;
    NSValue *shapeFunctionValue;
    pathfunction *shapeFunction;
    void *f = [[[entityTypePopUp selectedItem] representedObject] pathFunction];
    
//    [shapeMatrix removeRow:0];
//    [shapeMatrix renewRows:1 columns:0];
    [shapeMatrix renewRows:1 columns:[[[STEntityDescriptor shapeFunctionsValueEnumerator] allObjects] count]];
    shapeFunctionsValueEnumerator = [STEntityDescriptor shapeFunctionsValueEnumerator];
    while ((shapeFunctionValue = [shapeFunctionsValueEnumerator nextObject]) != NULL) {
        shapeFunction = (pathfunction *)[shapeFunctionValue pointerValue];
        imageRep = [[[MyCustomImageRep alloc] initWithDrawSelector:@selector(drawShape:) delegate:self] autorelease];
        [imageRep setFunction:shapeFunction];
        image = [NSImage allocWithZone:[self zone]];
        [[image initWithSize:size] autorelease];
#ifdef GNUSTEP
	[imageRep setSize:size];
#endif
        [image addRepresentation:imageRep];
        [image setBackgroundColor:[NSColor clearColor]];
//        [shapeMatrix addColumn];
        [[shapeMatrix cellAtRow:0 column:col] setImage:image];
//[[[shapeMatrix window] contentView] lockFocus];
//[image compositeToPoint:NSMakePoint(10,10) operation:NSCompositeCopy];
//[[[shapeMatrix window] contentView] unlockFocus];
//[[shapeMatrix window] flushWindow];
        if (f == shapeFunction)
            [shapeMatrix selectCellAtRow:0 column:col];
        col++;
    }
    [shapeMatrix sizeToCells];
}

- (void)setupDrawMatrixes
{
    MyCustomImageRep *imageRep;
    NSImage *image;
    NSSize size = [drawMatrix cellSize];
    NSEnumerator *drawFunctionsValueEnumerator;
    NSValue *drawFunctionValue;
    drawfunction *drawFunction;
    int col = 0;
    void *df = [[[entityTypePopUp selectedItem] representedObject] drawFunction];
    void *hf = [[[entityTypePopUp selectedItem] representedObject] highlightFunction];

//    [drawMatrix removeRow:0];
//    [drawMatrix renewRows:1 columns:0];
//    [highlightMatrix removeRow:0];
//    [highlightMatrix renewRows:1 columns:0];
    [drawMatrix renewRows:1 columns:[[[STEntityDescriptor drawFunctionsValueEnumerator] allObjects] count]];
    [highlightMatrix renewRows:1 columns:[[[STEntityDescriptor drawFunctionsValueEnumerator] allObjects] count]];
    drawFunctionsValueEnumerator = [STEntityDescriptor drawFunctionsValueEnumerator];
    while ((drawFunctionValue = [drawFunctionsValueEnumerator nextObject]) != NULL) {
        drawFunction = (drawfunction *)[drawFunctionValue pointerValue];
        imageRep = [[[MyCustomImageRep alloc] initWithDrawSelector:@selector(drawDraw:) delegate:self] autorelease];
        [imageRep setFunction:drawFunction];
        image = [NSImage allocWithZone:[self zone]];
        [[image initWithSize:size] autorelease];
#ifdef GNUSTEP
	[imageRep setSize:size];
#endif
        [image addRepresentation:imageRep];
        [image setBackgroundColor:[NSColor clearColor]];
//        [drawMatrix addColumn];
//        [highlightMatrix addColumn];
        [[drawMatrix cellAtRow:0 column:col] setImage:image];
        [[highlightMatrix cellAtRow:0 column:col] setImage:image];
        if (df == drawFunction)
            [drawMatrix selectCellAtRow:0 column:col];
        if (hf == drawFunction)
            [highlightMatrix selectCellAtRow:0 column:col];
        col++;
    }
    [drawMatrix sizeToCells];
    [highlightMatrix sizeToCells];
    //BUG - pourquoi est-ce que matrixes sont detruites?
    [drawMatrix retain];
    [highlightMatrix retain];
    [shapeMatrix retain];
}

- (void)drawShape:(MyCustomImageRep *)image
{
    NSSize size = [image size];
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    PajeDrawingType drawingType = [entityDescriptor drawingType];
    NSRect rect = [entityDescriptor drawingRect];
    pathfunction *path = [image function];
    drawfunction *draw = [(MyCustomImageRep *)[[[drawMatrix selectedCell] image] bestRepresentationForDevice:nil] function];

    if (draw == NULL) {
        draw = [[STEntityDescriptor class] drawFunctionForName:@"PSFillAndFrameBlack"];
    }

    switch (drawingType) {
        case PajeEventDrawingType :
            rect.origin.x += size.width / 2 - rect.size.width / 2;
            rect.origin.y += size.height / 2 - rect.size.height / 2;
            break;
        case PajeVariableDrawingType :
        case PajeStateDrawingType :
            rect.origin.x += 5;
            rect.origin.y += size.height / 2 - rect.size.height / 2;
            rect.size.width += size.width - 10;
            break;
        case PajeLinkDrawingType :
            rect.origin.x += 5;
            rect.origin.y += 5;
            rect.size.width += size.width - 10;
            rect.size.height += size.height - 10;
            break;
        case PajeContainerDrawingType :
            return;
        default:
            NSAssert1(0, @"Invalid drawing type %d", drawingType);
    }

    [[NSColor brownColor] set];
    PSgsave();
    path(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
    draw();
    PSgrestore();
}

- (void)drawDraw:(MyCustomImageRep *)image
{
    NSSize size = [image size];
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    PajeDrawingType drawingType = [entityDescriptor drawingType];
    NSRect rect = [entityDescriptor drawingRect];
    pathfunction *path = [(MyCustomImageRep *)[[[shapeMatrix selectedCell] image] bestRepresentationForDevice:nil] function];
    drawfunction *draw = [image function];

    if (path == NULL) {
        path = [[STEntityDescriptor class] shapeFunctionForName:@"PSRect"];
    }

    switch (drawingType) {
        case PajeEventDrawingType :
            rect.origin.x += size.width / 2;
            rect.origin.y += size.height / 2;
            break;
        case PajeVariableDrawingType :
        case PajeStateDrawingType :
            rect.origin.x += 3;
            rect.origin.y += size.height / 2;
            rect.size.width += size.width - 6;
            break;
        case PajeLinkDrawingType :
            rect.origin.x += 3;
            rect.origin.y += 3;
            rect.size.width += size.width - 6;
            rect.size.height += size.height - 6;
            break;
        case PajeContainerDrawingType :
            return;
        default:
            NSAssert1(0, @"Invalid drawing type %d", drawingType);
    }

    [[[NSColor brownColor] highlightWithLevel:.2] set];
    PSgsave();
    path(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
    draw();
    PSgrestore();
}

- (void)drawExampleView:(NSCustomImageRep *)image
{
    NSRect exrect, exrects[] = {
        NSMakeRect(5,0,30,20),
        NSMakeRect(35,0,50,-10),
        NSMakeRect(85,0,35,30),
    };
    NSColor *colors[] = {
        [NSColor brownColor],
        [NSColor orangeColor],
        [NSColor redColor],
        [NSColor blueColor],
    };
    int ci=0, cc = sizeof(colors)/sizeof(colors[0]);
    STEntityDescriptor *entityDescriptor;
    PajeDrawingType drawingType;
    NSRect rect;
    pathfunction *path;
    drawfunction *draw;
    drawfunction *high;
    int i, count, r, cr;
    float y = 5;

    count = [entityTypePopUp numberOfItems];

    for (i=0; i<count; i++) {
        entityDescriptor = [[entityTypePopUp itemAtIndex:i] representedObject];
        drawingType = [entityDescriptor drawingType];
        path = [entityDescriptor pathFunction];
        draw = [entityDescriptor drawFunction];
        high = [entityDescriptor highlightFunction];

        cr = sizeof(exrects) / sizeof(exrects[0]);
        for (r=0; r<cr; r++) {
            exrect = exrects[r];

            exrect.origin.y = y;

            rect = [entityDescriptor drawingRect];

            switch (drawingType) {
                case PajeEventDrawingType :
                    rect.origin.x += exrect.origin.x;
                    rect.origin.y += exrect.origin.y;
                    break;
                case PajeVariableDrawingType :
                case PajeStateDrawingType :
                    rect.origin.x += exrect.origin.x;
                    rect.origin.y += exrect.origin.y;
                    rect.size.width += exrect.size.width;
                    break;
                case PajeLinkDrawingType :
                    rect.origin.x += exrect.origin.x;
                    rect.origin.y += exrect.origin.y;
                    rect.size.width += exrect.size.width;
                    rect.size.height += exrect.size.height;
                    break;
                case PajeContainerDrawingType :
                    rect.origin.x += exrect.origin.x;
                    rect.origin.y += exrect.origin.y;
                    rect.size.width += exrect.size.width;
                    break;
                default:
                    NSAssert1(0, @"Invalid drawing type %d", drawingType);
            }

            PSgsave();
            ci++;
            if (ci >= cc) ci = 0;
            [colors[ci] set];
            path(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
            if (r == 1) high();
            else draw();
            PSgrestore();
        }
        y += NSMaxY([entityDescriptor drawingRect]);
    }
}

- (void)recacheAll
{
    int i, count, x;
    PajeEntity *selectedEntity;

    [shapeMatrix getNumberOfRows:&x columns:&count];
    for (i=0; i<count; i++)
        [[[shapeMatrix cellAtRow:0 column:i] image] recache];
    [shapeMatrix setNeedsDisplay:YES];
    [drawMatrix getNumberOfRows:&x columns:&count];
    for (i=0; i<count; i++)
        [[[drawMatrix cellAtRow:0 column:i] image] recache];
    [drawMatrix setNeedsDisplay:YES];
    [highlightMatrix getNumberOfRows:&x columns:&count];
    for (i=0; i<count; i++)
        [[[highlightMatrix cellAtRow:0 column:i] image] recache];
    [highlightMatrix setNeedsDisplay:YES];
    selectedEntity = [[entityTypePopUp selectedItem] representedObject];
    [delegate dataChangedForEntityType:[selectedEntity entityType]];
}

- (void)drawFunctionSelected:(id)sender
{
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    drawfunction *draw = [(MyCustomImageRep *)[[[drawMatrix selectedCell] image] bestRepresentationForDevice:nil] function];
    [entityDescriptor setDrawFunction:draw];
    [self recacheAll];
}

- (void)entityTypeSelected:(id)sender
{
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    NSRect rect = [entityDescriptor drawingRect];
    PajeDrawingType drawingType = [entityDescriptor drawingType];
    [separationField setFloatValue:NSMinY(rect)];
    [separationField setEnabled:(drawingType != PajeLinkDrawingType)];
    [heightField setFloatValue:NSHeight(rect)];
    [heightField setEnabled:(/*(drawingType != PajeContainerDrawingType)
                             && */(drawingType != PajeLinkDrawingType))];
    [widthField setFloatValue:NSWidth(rect)];
    [widthField setEnabled:((drawingType == PajeEventDrawingType)
                            || (drawingType == PajeLinkDrawingType)
                            || (drawingType == PajeVariableDrawingType))];
    [displacementField setFloatValue:NSMinX(rect)];
    [displacementField setEnabled:(drawingType == PajeEventDrawingType)];

    [self setupShapeMatrix];
    [self setupDrawMatrixes];
//    [self recacheAll];
}

- (void)highlightFunctionSelected:(id)sender
{
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    drawfunction *highlight = [(MyCustomImageRep *)[[[highlightMatrix selectedCell] image] bestRepresentationForDevice:nil] function];
    [entityDescriptor setHighlightFunction:highlight];
    [self recacheAll];
}

- (void)shapeSelected:(id)sender
{
    STEntityDescriptor *entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    pathfunction *path = [(MyCustomImageRep *)[[[shapeMatrix selectedCell] image] bestRepresentationForDevice:nil] function];
    [entityDescriptor setPathFunction:path];
    [self recacheAll];
}

- (void)sizeChanged:(id)sender
{
    STEntityDescriptor *entityDescriptor;
    NSRect rect;
    entityDescriptor = [[entityTypePopUp selectedItem] representedObject];
    rect = NSMakeRect([displacementField floatValue],
                      [separationField floatValue],
                      [widthField floatValue], [heightField floatValue]);
    [entityDescriptor setDrawingRect:rect];
    [self recacheAll];
}

@end
