/* PSOperators.h - Drawing engine operators that use default context

   Copyright (C) 1999 Free Software Foundation, Inc.
   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
   Based on code by Adam Fedor
   Date: Feb 1999
   
   This file is part of the GNU Objective C User Interface library.
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
   */

#ifndef _PSOperators_h_INCLUDE
#define _PSOperators_h_INCLUDE



#ifndef NO_GNUSTEP
#define	DEFCTXT	GSCurrentContext();
#else
#define	DEFCTXT	[NSGraphicsContext currentContext]
#endif

/* ----------------------------------------------------------------------- */
/* Color operations */
/* ----------------------------------------------------------------------- */
void
PScurrentcmykcolor(float *c, float *m, float *y, float *k);

void
PSsetcmykcolor(float c, float m, float y, float k);

/* ----------------------------------------------------------------------- */
/* Data operations */
/* ----------------------------------------------------------------------- */
void
PSclear();

void
PScleartomark();

void
PScopy(int n);

void
PScount(int *n);

void
PScounttomark(int *n);

void
PSdup();

void
PSexch();

void
PSexecstack();

void
PSget();

void
PSindex(int i);

void
PSmark();

void
PSmatrix();

void
PSnull();

void
PSpop();

void
PSput();

void
PSroll(int n, int j);

/* ----------------------------------------------------------------------- */
/* Font operations */
/* ----------------------------------------------------------------------- */
void
PSFontDirectory();

void
PSISOLatin1Encoding();

void
PSSharedFontDirectory();

void
PSStandardEncoding();

void
PScurrentcacheparams();

void
PScurrentfont();

void
PSdefinefont();

void
PSfindfont(const char *name);

void
PSmakefont();

void
PSscalefont(float size);

void
PSselectfont(const char *name, float scale);

void
PSsetfont(int f);

void
PSundefinefont(const char *name);

/* ----------------------------------------------------------------------- */
/* Gstate operations */
/* ----------------------------------------------------------------------- */
void
PSconcat(const float m[]);

void
PScurrentdash();

void
PScurrentflat(float *flatness);

void
PScurrentgray(float *gray);

void
PScurrentgstate(int gst);

void
PScurrenthalftone();

void
PScurrenthalftonephase(float *x, float *y);

void
PScurrenthsbcolor(float *h, float *s, float *b);

void
PScurrentlinecap(int *linecap);

void
PScurrentlinejoin(int *linejoin);

void
PScurrentlinewidth(float *width);

void
PScurrentmatrix();

void
PScurrentmiterlimit(float *limit);

void
PScurrentpoint(float *x, float *y);

void
PScurrentrgbcolor(float *r, float *g, float *b);

void
PScurrentscreen();

void
PScurrentstrokeadjust(int *b);

void
PScurrenttransfer();

void
PSdefaultmatrix();

void
PSgrestore();

void
PSgrestoreall();

void
PSgsave();

void
PSgstate();

void
PSinitgraphics();

void
PSinitmatrix();

void
PSrotate(float angle);

void
PSscale(float x, float y);

void
PSsetdash(const float pat[], int size, float offset);

void
PSsetflat(float flatness);

void
PSsetgray(float gray);

void
PSsetgstate(int gst);

void
PSsethalftone();

void
PSsethalftonephase(float x, float y);

void
PSsethsbcolor(float h, float s, float b);

void
PSsetlinecap(int linecap);

void
PSsetlinejoin(int linejoin);

void
PSsetlinewidth(float width);

void
PSsetmatrix();

void
PSsetmiterlimit(float limit);

void
PSsetrgbcolor(float r, float g, float b);

void
PSsetscreen();

void
PSsetstrokeadjust(int b);

void
PSsettransfer();

void
PStranslate(float x, float y);

/* ----------------------------------------------------------------------- */
/* I/O Operations operations */
/* ----------------------------------------------------------------------- */
void
PSflush();

/* ----------------------------------------------------------------------- */
/* Matrix operations */
/* ----------------------------------------------------------------------- */
void
PSconcatmatrix();

void
PSdtransform(float x1, float y1, float *x2, float *y2);

void
PSidentmatrix();

void
PSidtransform(float x1, float y1, float *x2, float *y2);

void
PSinvertmatrix();

void
PSitransform(float x1, float y1, float *x2, float *y2);

void
PStransform(float x1, float y1, float *x2, float *y2);

/* ----------------------------------------------------------------------- */
/* Opstack operations */
/* ----------------------------------------------------------------------- */
void
PSdefineuserobject();

void
PSexecuserobject(int index);

void
PSundefineuserobject(int index);

void
PSgetboolean(int *it);

void
PSgetchararray(int size, char s[]);

void
PSgetfloat(float *it);

void
PSgetfloatarray(int size, float a[]);

void
PSgetint(int *it);

void
PSgetintarray(int size, int a[]);

void
PSgetstring(char *s);

void
PSsendboolean(int it);

void
PSsendchararray(const char s[], int size);

void
PSsendfloat(float it);

void
PSsendfloatarray(const float a[], int size);

void
PSsendint(int it);

void
PSsendintarray(const int a[], int size);

void
PSsendstring(const char *s);

/* ----------------------------------------------------------------------- */
/* Paint operations */
/* ----------------------------------------------------------------------- */
void
PSashow(float x, float y, const char *s);

void
PSawidthshow(float cx, float cy, int c, float ax, float ay, const char *s);

void
PScopypage();

void
PSeofill();

void
PSerasepage();

void
PSfill();

void
PSimage();

void
PSimagemask();

void
PScolorimage();

void
PSalphaimage();

void
PSkshow(const char *s);

void
PSrectfill(float x, float y, float w, float h);

void
PSrectstroke(float x, float y, float w, float h);

void
PSshow(const char *s);

void
PSshowpage();

void
PSstroke();

void
PSstrokepath();

void
PSueofill(const char nums[], int n, const char ops[], int l);

void
PSufill(const char nums[], int n, const char ops[], int l);

void
PSustroke(const char nums[], int n, const char ops[], int l);

void
PSustrokepath(const char nums[], int n, const char ops[], int l);

void
PSwidthshow(float x, float y, int c, const char *s);

void
PSxshow(const char *s, const float numarray[], int size);

void
PSxyshow(const char *s, const float numarray[], int size);

void
PSyshow(const char *s, const float numarray[], int size);

/* ----------------------------------------------------------------------- */
/* Path operations */
/* ----------------------------------------------------------------------- */
void
PSarc(float x, float y, float r, float angle1, float angle2);

void
PSarcn(float x, float y, float r, float angle1, float angle2);

void
PSarct(float x1, float y1, float x2, float y2, float r);

void
PSarcto(float x1, float y1, float x2, float y2, float r, float *xt1, float *yt1, float *xt2, float *yt2);

void
PScharpath(const char *s, int b);

void
PSclip();

void
PSclippath();

void
PSclosepath();

void
PScurveto(float x1, float y1, float x2, float y2, float x3, float y3);

void
PSeoclip();

void
PSeoviewclip();

void
PSflattenpath();

void
PSinitclip();

void
PSinitviewclip();

void
PSlineto(float x, float y);

void
PSmoveto(float x, float y);

void
PSnewpath();

void
PSpathbbox(float *llx, float *lly, float *urx, float *ury);

void
PSpathforall();

void
PSrcurveto(float x1, float y1, float x2, float y2, float x3, float y3);

void
PSrectclip(float x, float y, float w, float h);

void
PSrectviewclip(float x, float y, float w, float h);

void
PSreversepath();

void
PSrlineto(float x, float y);

void
PSrmoveto(float x, float y);

void
PSsetbbox(float llx, float lly, float urx, float ury);

void
PSviewclip();

void
PSviewclippath();

/* ----------------------------------------------------------------------- */
/* Window system ops */
/* ----------------------------------------------------------------------- */
void
PScurrentdrawingfunction(int *function);

void
PScurrentgcdrawable(void* *gc, void* *draw, int *x, int *y);

void
PScurrentgcdrawablecolor(void* *gc, void* *draw, int *x, int *y, int colorInfo[]);

void
PScurrentoffset(int *x, int *y);

void
PSsetdrawingfunction(int function);

void
PSsetgcdrawable(void* gc, void* draw, int x, int y);

void
PSsetgcdrawablecolor(void* gc, void* draw, int x, int y, const int colorInfo[]);

void
PSsetoffset(short int x, short int y);

void
PSsetrgbactual(double r, double g, double b, int *success);

void
PScapturegstate(int *gst);

/*-------------------------------------------------------------------------*/
/* Graphics Extension Ops */
/*-------------------------------------------------------------------------*/

void 
PScomposite(float x, float y, float w, float h, int gstateNum, float dx, float dy, int op);

void 
PScompositerect(float x, float y, float w, float h, int op);

void 
PSdissolve(float x, float y, float w, float h, int gstateNum, float dx, float dy, float delta);

void 
PSreadimage( void );

void 
PSsetalpha(float a);

void 
PScurrentalpha(float *a);

/* ----------------------------------------------------------------------- */
/* GNUstep Event and other I/O extensions */
/* ----------------------------------------------------------------------- */
NSEvent*
PSGetEvent(unsigned mask, NSDate* limit, NSString *mode);

NSEvent*
PSPeekEvent(unsigned mask, NSDate* limit, NSString *mode);

void
PSDiscardEvents(unsigned mask, NSEvent* limit);

void
PSPostEvent(NSEvent* anEvent, BOOL atStart);

void
PSmouselocation(float *x, float *y);

void
PSsetinputstate(int window, int state);

/* ----------------------------------------------------------------------- */
/* Color operations */
/* ----------------------------------------------------------------------- */
void
PScurrentcmykcolor(float *c, float *m, float *y, float *k);

void
PSsetcmykcolor(float c, float m, float y, float k);

/* ----------------------------------------------------------------------- */
/* Data operations */
/* ----------------------------------------------------------------------- */
void
PSclear();

void
PScleartomark();

void
PScopy(int n);

void
PScount(int *n);

void
PScounttomark(int *n);

void
PSdup();

void
PSexch();

void
PSexecstack();

void
PSget();

void
PSindex(int i);

void
PSmark();

void
PSmatrix();

void
PSnull();

void
PSpop();

void
PSput();

void
PSroll(int n, int j);

/* ----------------------------------------------------------------------- */
/* Font operations */
/* ----------------------------------------------------------------------- */
void
PSFontDirectory();

void
PSISOLatin1Encoding();

void
PSSharedFontDirectory();

void
PSStandardEncoding();

void
PScurrentcacheparams();

void
PScurrentfont();

void
PSdefinefont();

void
PSfindfont(const char *name);

void
PSmakefont();

void
PSscalefont(float size);

void
PSselectfont(const char *name, float scale);

void
PSsetfont(int f);

void
PSundefinefont(const char *name);

/* ----------------------------------------------------------------------- */
/* Gstate operations */
/* ----------------------------------------------------------------------- */
void
PSconcat(const float m[]);

void
PScurrentdash();

void
PScurrentflat(float *flatness);

void
PScurrentgray(float *gray);

void
PScurrentgstate(int gst);

void
PScurrenthalftone();

void
PScurrenthalftonephase(float *x, float *y);

void
PScurrenthsbcolor(float *h, float *s, float *b);

void
PScurrentlinecap(int *linecap);

void
PScurrentlinejoin(int *linejoin);

void
PScurrentlinewidth(float *width);

void
PScurrentmatrix();

void
PScurrentmiterlimit(float *limit);

void
PScurrentpoint(float *x, float *y);

void
PScurrentrgbcolor(float *r, float *g, float *b);

void
PScurrentscreen();

void
PScurrentstrokeadjust(int *b);

void
PScurrenttransfer();

void
PSdefaultmatrix();

void
PSgrestore();

void
PSgrestoreall();

void
PSgsave();

void
PSgstate();

void
PSinitgraphics();

void
PSinitmatrix();

void
PSrotate(float angle);

void
PSscale(float x, float y);

void
PSsetdash(const float pat[], int size, float offset);

void
PSsetflat(float flatness);

void
PSsetgray(float gray);

void
PSsetgstate(int gst);

void
PSsethalftone();

void
PSsethalftonephase(float x, float y);

void
PSsethsbcolor(float h, float s, float b);

void
PSsetlinecap(int linecap);

void
PSsetlinejoin(int linejoin);

void
PSsetlinewidth(float width);

void
PSsetmatrix();

void
PSsetmiterlimit(float limit);

void
PSsetrgbcolor(float r, float g, float b);

void
PSsetscreen();

void
PSsetstrokeadjust(int b);

void
PSsettransfer();

void
PStranslate(float x, float y);

/* ----------------------------------------------------------------------- */
/* I/O Operations operations */
/* ----------------------------------------------------------------------- */
void
PSflush();

/* ----------------------------------------------------------------------- */
/* Matrix operations */
/* ----------------------------------------------------------------------- */
void
PSconcatmatrix();

void
PSdtransform(float x1, float y1, float *x2, float *y2);

void
PSidentmatrix();

void
PSidtransform(float x1, float y1, float *x2, float *y2);

void
PSinvertmatrix();

void
PSitransform(float x1, float y1, float *x2, float *y2);

void
PStransform(float x1, float y1, float *x2, float *y2);

/* ----------------------------------------------------------------------- */
/* Opstack operations */
/* ----------------------------------------------------------------------- */
void
PSdefineuserobject();

void
PSexecuserobject(int index);

void
PSundefineuserobject(int index);

void
PSgetboolean(int *it);

void
PSgetchararray(int size, char s[]);

void
PSgetfloat(float *it);

void
PSgetfloatarray(int size, float a[]);

void
PSgetint(int *it);

void
PSgetintarray(int size, int a[]);

void
PSgetstring(char *s);

void
PSsendboolean(int it);

void
PSsendchararray(const char s[], int size);

void
PSsendfloat(float it);

void
PSsendfloatarray(const float a[], int size);

void
PSsendint(int it);

void
PSsendintarray(const int a[], int size);

void
PSsendstring(const char *s);

/* ----------------------------------------------------------------------- */
/* Paint operations */
/* ----------------------------------------------------------------------- */
void
PSashow(float x, float y, const char *s);

void
PSawidthshow(float cx, float cy, int c, float ax, float ay, const char *s);

void
PScopypage();

void
PSeofill();

void
PSerasepage();

void
PSfill();

void
PSimage();

void
PSimagemask();

void
PScolorimage();

void
PSalphaimage();

void
PSkshow(const char *s);

void
PSrectfill(float x, float y, float w, float h);

void
PSrectstroke(float x, float y, float w, float h);

void
PSshow(const char *s);

void
PSshowpage();

void
PSstroke();

void
PSstrokepath();

void
PSueofill(const char nums[], int n, const char ops[], int l);

void
PSufill(const char nums[], int n, const char ops[], int l);

void
PSustroke(const char nums[], int n, const char ops[], int l);

void
PSustrokepath(const char nums[], int n, const char ops[], int l);

void
PSwidthshow(float x, float y, int c, const char *s);

void
PSxshow(const char *s, const float numarray[], int size);

void
PSxyshow(const char *s, const float numarray[], int size);

void
PSyshow(const char *s, const float numarray[], int size);

/* ----------------------------------------------------------------------- */
/* Path operations */
/* ----------------------------------------------------------------------- */
void
PSarc(float x, float y, float r, float angle1, float angle2);

void
PSarcn(float x, float y, float r, float angle1, float angle2);

void
PSarct(float x1, float y1, float x2, float y2, float r);

void
PSarcto(float x1, float y1, float x2, float y2, float r, float *xt1, float *yt1, float *xt2, float *yt2);

void
PScharpath(const char *s, int b);

void
PSclip();

void
PSclippath();

void
PSclosepath();

void
PScurveto(float x1, float y1, float x2, float y2, float x3, float y3);

void
PSeoclip();

void
PSeoviewclip();

void
PSflattenpath();

void
PSinitclip();

void
PSinitviewclip();

void
PSlineto(float x, float y);

void
PSmoveto(float x, float y);

void
PSnewpath();

void
PSpathbbox(float *llx, float *lly, float *urx, float *ury);

void
PSpathforall();

void
PSrcurveto(float x1, float y1, float x2, float y2, float x3, float y3);

void
PSrectclip(float x, float y, float w, float h);

void
PSrectviewclip(float x, float y, float w, float h);

void
PSreversepath();

void
PSrlineto(float x, float y);

void
PSrmoveto(float x, float y);

void
PSsetbbox(float llx, float lly, float urx, float ury);

void
PSviewclip();

void
PSviewclippath();

/* ----------------------------------------------------------------------- */
/* Window system ops */
/* ----------------------------------------------------------------------- */
void
PScurrentdrawingfunction(int *function);

void
PScurrentgcdrawable(void* *gc, void* *draw, int *x, int *y);

void
PScurrentgcdrawablecolor(void* *gc, void* *draw, int *x, int *y, int colorInfo[]);

void
PScurrentoffset(int *x, int *y);

void
PSsetdrawingfunction(int function);

void
PSsetgcdrawable(void* gc, void* draw, int x, int y);

void
PSsetgcdrawablecolor(void* gc, void* draw, int x, int y, const int colorInfo[]);

void
PSsetoffset(short int x, short int y);

void
PSsetrgbactual(double r, double g, double b, int *success);

void
PScapturegstate(int *gst);

/*-------------------------------------------------------------------------*/
/* Graphics Extension Ops */
/*-------------------------------------------------------------------------*/

void 
PScomposite(float x, float y, float w, float h, int gstateNum, float dx, float dy, int op);

void 
PScompositerect(float x, float y, float w, float h, int op);

void 
PSdissolve(float x, float y, float w, float h, int gstateNum, float dx, float dy, float delta);

void 
PSreadimage( void );

void 
PSsetalpha(float a);

void 
PScurrentalpha(float *a);

/* ----------------------------------------------------------------------- */
/* GNUstep Event and other I/O extensions */
/* ----------------------------------------------------------------------- */
NSEvent*
PSGetEvent(unsigned mask, NSDate* limit, NSString *mode);

NSEvent*
PSPeekEvent(unsigned mask, NSDate* limit, NSString *mode);

void
PSDiscardEvents(unsigned mask, NSEvent* limit);

void
PSPostEvent(NSEvent* anEvent, BOOL atStart);

void
PSmouselocation(float *x, float *y);

void
PSsetinputstate(int window, int state);

#endif	
