/*
 */

#include "STController.h"
#include "DrawView.h"
#include "EntityTypeInspector.h"

@implementation STController

- (id)initWithController:(PajeTraceController *)c
{
    [super initWithController:c];

    if (![NSBundle loadNibNamed:@"SpaceTime" owner:self])
        NSRunAlertPanel(@"SpaceTime", @"Couldn't load interface file",
                        nil, nil, nil);
//    if (![NSBundle loadNibNamed:@"LotsOfColors" owner:self])
//        NSRunAlertPanel(@"SpaceTime", @"Couldn't load interface file LotsOfColors",
//                        nil, nil, nil);

    return self;
}

- (void)awakeFromNib
{
    // set rulers
    [scrollView setHasVerticalRuler:YES];
    hierarchyRuler = [[HierarchyRuler alloc] initWithScrollView:scrollView];
    [scrollView setVerticalRulerView:hierarchyRuler];
    [hierarchyRuler setClientView:drawView];

    [scrollView setHasHorizontalRuler:/*NO];*/YES];
    [[scrollView horizontalRulerView] setClientView:drawView];
    [[scrollView horizontalRulerView] setReservedThicknessForMarkers:0.0];

    [scrollView setRulersVisible:NO/*YES*/];

    [window setDelegate:self];
    [window setFrameAutosaveName:@"SpaceTime"];
    [window makeKeyAndOrderFront:self];

    // register the tools with the controller
    [self registerTool:self];
    [self registerTool:[[EntityTypeInspector alloc] initWithDelegate:drawView]];
}

- (void)windowDidBecomeKey:(NSNotification *)notification
{
    // tell controller that we are key (to change filter menus)
    // TODO: find a cleaner way of doing this
    [controller windowIsKey];
}

- (void)windowDidResize:(NSNotification *)notification
{
    [drawView windowDidResize:notification];
}

- (NSString *)toolName
{
    return @"Space Time Diagram";
}

- (void)setInputComponent:(PajeComponent *)filter
{
    [super setInputComponent:filter];
    [drawView setFilter:(PajeFilter *)filter];
}

- (void)hierarchyChanged
{
//NSLog(@"rep: %@", [self nameForEntity:[self rootInstance]]);
    [window setTitleWithRepresentedFilename:[self nameForEntity:[self rootInstance]]];
    [drawView hierarchyChanged];
    [hierarchyRuler refreshSizes];
    [scrollView setRulersVisible:YES];
}

- (void)dataChangedForEntityType:(PajeEntityType *)entityType
{
    [drawView dataChangedForEntityType:entityType];
}

- (void)colorChangedForEntityType:(PajeEntityType *)entityType
{
    [drawView colorChangedForEntityType:entityType];
}

- (void)orderChangedForContainerType:(PajeContainerType *)containerType;
{
    [drawView orderChangedForContainerType:containerType];
}

- (void)timeSelectionChanged
{
    [drawView setNeedsDisplay:YES];
}

- (void)containerSelectionChanged
{
    [drawView setNeedsDisplay:YES];
    [hierarchyRuler setNeedsDisplay:YES];
}

- (void)fileSelected:(NSNotification *)notification
/* receives PajeFilenameNotification, changes the window title accordingly */
{
    [window setTitleWithRepresentedFilename:[[notification userInfo] objectForKey:@"Filename"]];
}

- (void)activateTool:(id)sender
/* sent by PajeController when the user selects this Tool */
{
    [window makeKeyAndOrderFront:self];
}

- (void)print:(id)sender
{
//    [[NSPrintOperation printOperationWithView:[window contentView]] runOperation]; 
    [window print:sender];
}
@end
