/* PieCell.m created by benhur on Wed 24-Sep-1997 */

#import "PieCell.h"
#include <math.h>

static NSString *formatNumber(double n)
{
    static char *p = "TGMk munp";
    int i = 4;
    BOOL neg = (n < 0);

    if (neg)
        n = -n;
    if (n < 1e-12)
        return neg ? @"-0" : @"0";
    if (n > 999e12)
        return neg ? @"-Inf" : @"Inf";
    while (n < 1) {
        n *= 1000;
        i++;
    }
    while (n > 1000) {
        n /= 1000;
        i--;
    }
    if (n < 10)
        return [NSString stringWithFormat:@"%s%3.1f%c", neg?"-":"", n, p[i]];
    return [NSString stringWithFormat:@"%s%3d%c", neg?"-":"", (int)n, p[i]];
}


@interface NSString (PajeNSStringPositionDrawing)
// additions to NSString to support drawing them in many positions
// relative to a given point. Uses NSStringAdditions methods to do this.
// Should not be called if the focus is not locked to some view.
// All methods are of the form: drawAtXYPoint: ...,
// where X can be L, C or R, meaning that the x coordinate of aPoint
// should be at the left, center or right of the string, and
// Y can be B, C or T (bottom, center or top)
- (void)drawAtLTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtLCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtLBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtCTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtCCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtCBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtRTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtRCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
- (void)drawAtRBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes;
@end

@implementation NSString (PajeNSStringPositionDrawing)
- (void)drawAtLTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    [self drawAtPoint:aPoint withAttributes:attributes];
}
- (void)drawAtLCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x, aPoint.y - size.height/2);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtLBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x, aPoint.y - size.height);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtCTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width/2, aPoint.y);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtCCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width/2, aPoint.y - size.height/2);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtCBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width/2, aPoint.y - size.height);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtRTPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width, aPoint.y);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtRCPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width, aPoint.y - size.height/2);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
- (void)drawAtRBPoint:(NSPoint)aPoint withAttributes:(NSDictionary *)attributes
{
    NSSize size = [self sizeWithAttributes:attributes];
    NSPoint newPoint = NSMakePoint(aPoint.x - size.width, aPoint.y - size.height);
    [self drawAtPoint:newPoint withAttributes:attributes];
}
@end

void PSleftshow(const char *s)
{
    float dx, dy;
    PSstringwidth(s, &dx, &dy);
    PSrmoveto(-dx, -dy);
    PSshow(s);
}

void PScentershow(const char *s)
{
    float dx, dy;
    PSstringwidth(s, &dx, &dy);
    PSrmoveto(-dx/2, -dy/2);
    PSshow(s);
}


@implementation PieCell
- (void)setDataProvider:(id)provider
{
    dataProvider = provider;
}

- (void)setData:(StatArray *)d
{
    [d retain];
    [data release];
    data = d;
}

- (void)setInitialAngle:(NSNumber *)angle
{
    initialAngle = angle;
}

- (void)drawInteriorWithFrameBar:(NSRect)cellFrame inView:(NSView *)controlView
{
    id value;
    NSEnumerator *valEnum;
    float x, y, h, hmax, w, wmax;
    float val, total, max, min, cnt;
    float fontHeight = 0;
    NSFont *font = nil;

    if (nil == data) {
        [dataProvider provideDataForCell:self];
        if (nil == data)
            return;
    }

    NSFrameRect(cellFrame);

    // take totalizers
    total = [data sum];
    max = [data maxValue];
    min = [data minValue];
    cnt = [data count];

    PSsetlinewidth(1);

    if (!simple) {
        // draw title
        font = [NSFont userFontOfSize:12];
        [font set];
        fontHeight = [font ascender] - [font descender];
        PSmoveto(NSMidX(cellFrame), NSMinY(cellFrame) + [font ascender]+2);
        PScentershow([[data name] cString]);

        // calculate sizes, with space for title and names
        hmax = NSHeight(cellFrame) - 4 - 35 - (fontHeight + 2);
        wmax = NSWidth(cellFrame) - 4 - 30;
        y = NSMaxY(cellFrame) - 2 - 35;
        w = wmax / cnt;
        x = NSMaxX(cellFrame) - 2 - wmax;

        font = [NSFont userFontOfSize:10];
        [font set];
        fontHeight = [font ascender] - [font descender];

        // draw scale
        PSmoveto(x-2, y);
        PSrlineto(-3, 0);
        PSrmoveto(-2, 3);
        PSleftshow("0");
        PSmoveto(x-2, y-hmax);
        PSrlineto(-3, 0);
        PSrmoveto(-2, 3);
        PSleftshow([formatNumber(max)/*[[NSNumber numberWithFloat:max] stringValue]*/ cString]);
        PSstroke();
   } else {
       // calculate sizes, without space for title and names
        hmax = NSHeight(cellFrame) - 4;
        wmax = NSWidth(cellFrame) - 4;
        y = NSMaxY(cellFrame) - 2;
        w = wmax / cnt;
        x = NSMaxX(cellFrame) - 2 - wmax;
    }

    valEnum = [data objectEnumerator];
    while (value = [valEnum nextObject]) {
        val = [value doubleValue];

        // draw bar
        [[value color] set];
        h = hmax * val / max;
        PSrectfill(x, y, w, -h);
        [[NSColor blackColor] set];
        PSrectstroke(x, y, w, -h);

        if (!simple) {
            // draw name under bar
            PSgsave();
            PSmoveto(x + w/2, y);
            if (w > fontHeight) {
                PSrotate(-atan(fontHeight / sqrt(w*w + fontHeight*fontHeight))*180/M_PI);
            } else {
                PSrotate(-90);
            }
            PSrmoveto(0, [font ascender]);
            PSleftshow([[value name] cString]);
            PSgrestore();
        }

        x += w;
    }
}

- (void)drawInteriorWithFrameBar2:(NSRect)cellFrame inView:(NSView *)controlView
{
    id value;
    NSEnumerator *valEnum;
    float x, y, h, hmax, w, wmax;
    float val, total, max, min, cnt;
    float fontHeight = 0;
    NSFont *font = nil;

    if (nil == data) {
        [dataProvider provideDataForCell:self];
        if (nil == data)
            return;
    }

    NSFrameRect(cellFrame);

    // take totalizers
    total = [data sum];
    max = [data maxValue];
    min = [data minValue];
    cnt = [data count];

    PSsetlinewidth(1);

    if (!simple) {
        // draw title
        font = [NSFont userFontOfSize:12];
        [font set];
        fontHeight = [font ascender] - [font descender];
        PSmoveto(NSMidX(cellFrame), NSMinY(cellFrame) + [font ascender]+2);
        PScentershow([[data name] cString]);

        // calculate sizes, with space for title and names
        hmax = NSHeight(cellFrame) - 4 - 35 - (fontHeight + 2);
        wmax = NSWidth(cellFrame) - 4 - 30;
        y = NSMaxY(cellFrame) - 2 - 35;
        w = wmax / cnt;
        x = NSMaxX(cellFrame) - 2 - wmax;

        font = [NSFont userFontOfSize:10];
        [font set];
        fontHeight = [font ascender] - [font descender];

        // draw scale
        PSmoveto(x-2, y);
        PSrlineto(-3, 0);
        PSrmoveto(-2, 3);
        PSleftshow([formatNumber(min)/*[[NSNumber numberWithFloat:min] stringValue]*/ cString]);
        PSmoveto(x-2, y-hmax);
        PSrlineto(-3, 0);
        PSrmoveto(-2, 3);
        PSleftshow([formatNumber(max)/*[[NSNumber numberWithFloat:max] stringValue]*/ cString]);
        PSstroke();
   } else {
       // calculate sizes, without space for title and names
        hmax = NSHeight(cellFrame) - 4;
        wmax = NSWidth(cellFrame) - 4;
        y = NSMaxY(cellFrame) - 2;
        w = wmax / cnt;
        x = NSMaxX(cellFrame) - 2 - wmax;
    }

    valEnum = [data objectEnumerator];
    while (value = [valEnum nextObject]) {
        val = [value doubleValue];

        // draw bar
        [[value color] set];
        h = hmax * (val - min) / (max - min);
        PSrectfill(x, y, w, -h);
        [[NSColor blackColor] set];
        PSrectstroke(x, y, w, -h);

        if (!simple) {
            // draw name under bar
            PSgsave();
            PSmoveto(x + w/2, y);
            if (w > fontHeight) {
                PSrotate(-atan(fontHeight / sqrt(w*w + fontHeight*fontHeight))*180/M_PI);
            } else {
                PSrotate(-90);
            }
            PSrmoveto(0, [font ascender]);
            PSleftshow([[value name] cString]);
            PSgrestore();
        }

        x += w;
    }
}

- (void)drawInteriorWithFramePie:(NSRect)cellFrame inView:(NSView *)controlView
{
    id value;
    NSEnumerator *valEnum;
    float radius, angle1 = 0, angle2, mid_angle;
    NSPoint center;
    float val, total;
    float fontHeight = 0;
    NSFont *font = nil;
    NSMutableDictionary *attributes = [NSMutableDictionary dictionary];

    if (nil == data) {
        [dataProvider provideDataForCell:self];
        if (nil == data)
            return;
    }

    // take totalizers
    total = [data sum];
    if (total == 0)
        return;

    if (initialAngle) angle1 = [initialAngle floatValue];

    if (!simple) {
        // calculate center, radius, with space for anotation
        font = [[NSFontManager sharedFontManager] convertFont:[NSFont userFontOfSize:12] toHaveTrait:NSBoldFontMask];
        fontHeight = [font ascender] - [font descender];
        center = NSMakePoint(NSMidX(cellFrame),
                             NSMidY(cellFrame) + fontHeight/2 + 2);
        radius = MIN(NSWidth(cellFrame) * .5 - 20,
                     NSHeight(cellFrame) * .5 - 1 /*10*/ - fontHeight - 4);
        
        // write title
        [attributes setObject:font forKey:NSFontAttributeName];
        [attributes setObject:[NSColor blackColor] forKey:NSForegroundColorAttributeName];
        [[NSColor blackColor] set];
        [[data name] drawAtCTPoint:NSMakePoint(center.x, NSMinY(cellFrame))
                      withAttributes:attributes];

        // set font for slice names
        font = [NSFont userFontOfSize:10];
        [attributes setObject:[NSFont userFontOfSize:10] forKey:NSFontAttributeName];
        [font set];
        fontHeight = [font ascender] - [font descender];
    } else {
        center = NSMakePoint(NSMidX(cellFrame), NSMidY(cellFrame));
        radius = MIN(NSWidth(cellFrame) * .5 - 2, NSHeight(cellFrame) * .5 - 2);
    }

    [[NSColor blackColor] set];
    PSsetlinewidth(2);
    PSsetlinejoin(1); // rounded

    valEnum = [data objectEnumerator];
    while (value = [valEnum nextObject]) {
        val = [value doubleValue];
        angle2 = angle1 + 360 * val / total;

        if ((val/total) < .05) continue;

        // draw the pie segment and border
        PSnewpath();
        PSmoveto(center.x, center.y);
        PSarc(center.x, center.y, radius, angle1, angle2);
        PSclosepath();
        PSgsave();
        [[value color] set];
        PSfill();
        PSgrestore();
        PSstroke();

        if (!simple) {
            NSString *valueString;
            NSColor *sliceTextColor;
            NSPoint sliceCenter;

            // angle in the middle of the slice (in radians)
            mid_angle = (angle1 + angle2) / 2;
            if (mid_angle > 360) mid_angle -= 360;
            // convert to radians
            mid_angle = M_PI * mid_angle / 180;

            PSgsave();
            // write value inside slice
            // what to write
            if (showPercent)
                valueString = [NSString stringWithFormat:@"%.1f%%", 100*val/total];
            else
                valueString = formatNumber(val);//[NSString stringWithFormat:@"%.2f", val];
            // color to write (black or white, the one with better contrast)
            if ([[[value color] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent] > .3)
                sliceTextColor = [NSColor blackColor];
            else
                sliceTextColor = [NSColor whiteColor];
            [attributes setObject:sliceTextColor
                           forKey:NSForegroundColorAttributeName];
            // where to write
            sliceCenter = NSMakePoint(center.x+(radius*.7)*cos(mid_angle),
                                      center.y+(radius*.7)*sin(mid_angle));
            [valueString drawAtCCPoint:sliceCenter withAttributes:attributes];
            PSgrestore();

            // draw the name of the slice
            [attributes setObject:[NSColor blackColor]
                           forKey:NSForegroundColorAttributeName];
            PSgsave();
            PSmoveto(center.x+(radius*1.05)*cos(mid_angle),
                     center.y+(radius*1.05)*sin(mid_angle));
            PSlineto(center.x+(radius*1.15)*cos(mid_angle),
                     center.y+(radius*1.15)*sin(mid_angle));
            if (mid_angle > M_PI_2 && mid_angle < 3 * M_PI_2) {
                NSPoint point;
                point = NSMakePoint(center.x+(radius*1.15)*cos(mid_angle) - 12,
                                    center.y+(radius*1.15)*sin(mid_angle));
                PSrlineto(-10, 0);
                [[value name] drawAtRCPoint:point withAttributes:attributes];
            } else {
                NSPoint point;
                point = NSMakePoint(center.x+(radius*1.15)*cos(mid_angle) + 12,
                                    center.y+(radius*1.15)*sin(mid_angle));
                PSrlineto(10, 0);
                [[value name] drawAtLCPoint:point withAttributes:attributes];
            }
            PSsetlinewidth(1);
            PSstroke();
            PSgrestore();
        }

        // next segment starts where this one ends
        angle1 = angle2;
    }
}

- (void)drawInteriorWithFramePie2:(NSRect)cellFrame inView:(NSView *)controlView
{
    id value;
    NSEnumerator *valEnum;
    float radius, angle1 = 0, angle2, mid_angle;
    NSPoint center;
    float cx, cy;
    float val, total, max, min, cnt;
    int n = 0;
    float fontHeight = 0;
    NSFont *font = nil;

    if (nil == data) {
        [dataProvider provideDataForCell:self];
        if (nil == data)
            return;
    }

    if (initialAngle) angle1 = [initialAngle floatValue];

    // take totalizers
    total = [data sum];
    max = [data maxValue];
    min = [data minValue];
    cnt = [data count];

    if (!simple) {
        // calculate center, radius, with space for anotation
        font = [NSFont userFontOfSize:12];
        [font set];
        fontHeight = [font ascender] - [font descender];
        center = NSMakePoint(NSMidX(cellFrame),
                             NSMidY(cellFrame) + fontHeight/2 + 2);
        radius = MIN(NSWidth(cellFrame) * .5 - 20,
                     NSHeight(cellFrame) * .5 - 10 - fontHeight - 4);
        // write title
        PSmoveto(center.x, NSMinY(cellFrame) + fontHeight);
        PScentershow([[data name] cString]);

        // set font for slice names
        font = [NSFont userFontOfSize:10];
        [font set];
        fontHeight = [font ascender] - [font descender];
    } else {
        center = NSMakePoint(NSMidX(cellFrame), NSMidY(cellFrame));
        radius = MIN(NSWidth(cellFrame) * .5 - 2, NSHeight(cellFrame) * .5 - 2);
    }

    [[NSColor blackColor] set];
    PSsetlinewidth(2);

    valEnum = [data objectEnumerator];
    while (value = [valEnum nextObject]) {
        val = [value doubleValue];
        angle2 = angle1 + 360 / cnt;

        // draw the pie segment and border
        PSnewpath();
        PSmoveto(center.x, center.y);
        PSarc(center.x, center.y, radius * val / max, angle1, angle2);
        PSclosepath();
        PSgsave();
        [[value color] set];
        PSfill();
        PSgrestore();
        PSstroke();

        if (!simple) {
            // draw the name represented in segment
            mid_angle = (angle1 + angle2) / 2;
            if (mid_angle > 360) mid_angle -= 360;
            // convert to radians
            mid_angle = M_PI * mid_angle / 180;
            PSgsave();
            PSnewpath();
            PSmoveto(center.x+(radius*(val/max+.1))*cos(mid_angle),
                     center.y+(radius*(val/max+.1))*sin(mid_angle));
            PSlineto(center.x+(radius*1.1)*cos(mid_angle),
                     center.y+(radius*1.1)*sin(mid_angle));
            if (mid_angle > M_PI_2 && mid_angle < 3 * M_PI_2) {
//                PSrlineto(-10, 0);
                PScurrentpoint(&cx, &cy);
                PSlineto(center.x-radius-4, cy);
                PSrmoveto(-2, fontHeight/3);
                PSleftshow([[value name] cString]);
            } else {
//                PSrlineto(10, 0);
                PScurrentpoint(&cx, &cy);
                PSlineto(center.x+radius+4, cy);
                PSrmoveto(2, fontHeight/3);
                PSshow([[value name] cString]);
            }
//            PSlineto(x+r+4, y-r-4+n*15);
//            PSrmoveto(2, 5);
            PSclosepath();
            PSsetlinewidth(1);
            PSstroke();
            PSgrestore();
        }

        n++;
        // next segment starts where this one ends
        angle1 = angle2;
    }
}

- (void)drawInteriorWithFrameNova:(NSRect)cellFrame inView:(NSView *)controlView
{
    id value;
    NSEnumerator *valEnum;
    float radius, angle1 = 0, angle2, mid_angle;
    NSPoint center;
    float cx, cy;
    float val, total, max, min, cnt;
    int n = 0;
    float fontHeight = 0;
    NSFont *font = nil;

    if (nil == data) {
        [dataProvider provideDataForCell:self];
        if (nil == data)
            return;
    }

    if (initialAngle) angle1 = [initialAngle floatValue];

    // take totalizers
    total = [data sum];
    max = [data maxValue];
    min = [data minValue];
    cnt = [data count];

    if (!simple) {
        // calculate center, radius, with space for anotation
        font = [NSFont userFontOfSize:12];
        [font set];
        fontHeight = [font ascender] - [font descender];
        center = NSMakePoint(NSMidX(cellFrame),
                             NSMidY(cellFrame) + fontHeight/2 + 2);
        radius = MIN(NSWidth(cellFrame) * .5 - 20,
                     NSHeight(cellFrame) * .5 - 10 - fontHeight - 4);
        // write title
        PSmoveto(center.x, NSMinY(cellFrame) + fontHeight);
        PScentershow([[data name] cString]);

        // set font for slice names
        font = [NSFont userFontOfSize:10];
        [font set];
        fontHeight = [font ascender] - [font descender];
    } else {
        center = NSMakePoint(NSMidX(cellFrame), NSMidY(cellFrame));
        radius = MIN(NSWidth(cellFrame) * .5 - 2, NSHeight(cellFrame) * .5 - 2);
    }

    PSsetlinewidth(3);

    valEnum = [data objectEnumerator];
    while (value = [valEnum nextObject]) {
        val = [value doubleValue];
        angle2 = angle1 + 360 / cnt;

        // draw the pie segment and border
        PSnewpath();
        PSmoveto(center.x, center.y);
        PSarc(center.x, center.y, radius * val / max, angle1, angle1);
        PSclosepath();
        [[value color] set];
        PSstroke();

        if (!simple) {
            // draw the name represented in segment
            mid_angle = angle1;
            if (mid_angle > 360) mid_angle -= 360;
            // convert to radians
            mid_angle = M_PI * mid_angle / 180;
            PSgsave();
            PSnewpath();
            PSmoveto(center.x+(radius*(val/max+.1))*cos(mid_angle),
                     center.y+(radius*(val/max+.1))*sin(mid_angle));
            PSlineto(center.x+(radius*1.1)*cos(mid_angle),
                     center.y+(radius*1.1)*sin(mid_angle));
            if (mid_angle > M_PI_2 && mid_angle < 3 * M_PI_2) {
//                PSrlineto(-10, 0);
                PScurrentpoint(&cx, &cy);
                PSlineto(center.x-radius-4, cy);
                PSrmoveto(-2, fontHeight/3);
                PSleftshow([[value name] cString]);
            } else {
//                PSrlineto(10, 0);
                PScurrentpoint(&cx, &cy);
                PSlineto(center.x+radius+4, cy);
                PSrmoveto(2, fontHeight/3);
                PSshow([[value name] cString]);
            }
//            PSlineto(x+r+4, y-r-4+n*15);
//            PSrmoveto(2, 5);
            PSclosepath();
            PSsetlinewidth(1);
            [[NSColor blackColor] set];
            PSstroke();
            PSgrestore();
        }

        n++;
        // next segment starts where this one ends
        angle1 = angle2;
    }
}

- (void)setGraphType:(int)t
{
    type = t;
}

- (void)setShowPercent:(BOOL)yn
{
    showPercent = yn;
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    switch(type) {
        case 0: [self drawInteriorWithFramePie:cellFrame inView:controlView];
            break;
        case 1: [self drawInteriorWithFramePie2:cellFrame inView:controlView];
            break;
        case 2: [self drawInteriorWithFrameBar:cellFrame inView:controlView];
            break;
        case 3: [self drawInteriorWithFrameBar2:cellFrame inView:controlView];
            break;
        case 4: [self drawInteriorWithFrameNova:cellFrame inView:controlView];
            break;
        case 5: [self drawInteriorWithFrameNova:cellFrame inView:controlView];
            break;
    }
}

@end
